// BasisDoc.cpp : implementation of the CBasisDoc class
//
#include "OS.h"
#include "stdafx.h"
#include "Basis.h"

#include "BaseAgent.h"
#include "TViewingAgent.h"
#include "BasisDoc.h"
#include "BasisView.h"
#include	"TPerceptionView.h"
#include <stdlib.h>
#include <time.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc

IMPLEMENT_DYNCREATE(CBasisDoc, CDocument)

BEGIN_MESSAGE_MAP(CBasisDoc, CDocument)
	//{{AFX_MSG_MAP(CBasisDoc)
	ON_COMMAND(IDM_GHOSTTEAM, OnGhostteam)
	ON_COMMAND(IDM_GHOSTDISTRIBUTE, OnGhostdistribute)
	ON_COMMAND(ID_CONNECT, OnConnect)
	ON_COMMAND(ID_DISCONNECT, OnDisconnect)
	ON_UPDATE_COMMAND_UI(ID_DISCONNECT, OnUpdateDisconnect)
	ON_UPDATE_COMMAND_UI(ID_CONNECT, OnUpdateConnect)
	ON_COMMAND(ID_VIEW_NOGRAPHICS, OnViewNographics)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NOGRAPHICS, OnUpdateViewNographics)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc construction/destruction

CBasisDoc::CBasisDoc()
{
	int	i;

	m_pBasisView = NULL;
	m_pPerceptionView = NULL;
	m_pAgent = NULL;
	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;
	m_bNoGraphics = FALSE;

	for (i = 0; i < 11; i++)
		m_pGhost[i] = NULL;
}

CBasisDoc::~CBasisDoc()
{
	int	i;

	for (i = 0; i < 11; i++)
	{
		if (m_pGhost[i] != NULL)
		{
			m_pGhost[i]->DisconnectAgent();
			delete m_pGhost[i];
		}
	}
}

BOOL CBasisDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// get the pointers of all views
	for(POSITION pos = GetFirstViewPosition(); pos != NULL;)
	{
		CView* pView = GetNextView(pos);
		
		if (pView->IsKindOf(RUNTIME_CLASS(CBasisView)))
			m_pBasisView = (CBasisView*) pView;
		else if (pView->IsKindOf(RUNTIME_CLASS(TPerceptionView)))
			m_pPerceptionView = (TPerceptionView*) pView;
	}

	// set all views to display graphics or not
	m_pBasisView->SetNoGraphics(m_bNoGraphics);
	m_pPerceptionView->SetNoGraphics(m_bNoGraphics);

	// create a new agent
	m_pAgent = new TViewingAgent(m_pBasisView, m_pPerceptionView);
	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;

	// connect the agent to the server
//	OnConnect();
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CBasisDoc serialization

void CBasisDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CBasisDoc diagnostics

#ifdef _DEBUG
void CBasisDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CBasisDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CBasisDoc commands

void CBasisDoc::DeleteContents() 
{
	if (m_pAgent != NULL)
	{
		// delete the agent
		OnDisconnect();
		delete m_pAgent;
		m_pAgent = NULL;
	}

//	if (m_pBasisView != NULL)
//		m_pBasisView->GetEditCtrl().SetWindowText(_T(""));

	CDocument::DeleteContents();
	m_bConnected = FALSE;
	m_bTryingToConnect = FALSE;
}

void CBasisDoc::OnGhostteam() 
{
	int			i;
	
	// only do once
	if (m_pGhost[0] != NULL)
		return;

	for (i = 0; i < 11; i++)
	{
		
		// create a new agent
		if (m_pGhost[i] = new CBaseAgent());
		{
			// connect the server, init a player and move it a little
			m_pGhost[i]->ConnectAgent(_T("Ghost"), _T("ultra2"), 6000);
		}
	}
}

void CBasisDoc::OnGhostdistribute() 
{
	int	i;
	
   srand( (unsigned)time( NULL ) );
	for (i = 0; i < 11; i++)
		if (m_pGhost[i] != NULL)
			// distribute the ghost players across their field
			m_pGhost[i]->Move(-rand()/600, 30 - rand()/600);
}

void CBasisDoc::OnConnect() 
{
	// connect as player
	if (m_pAgent->Connect())
	{
		m_bTryingToConnect = TRUE;
	//	m_pView->StartConnectionTimer();
		m_bConnected = TRUE;
	}
}

void CBasisDoc::OnDisconnect() 
{
	// if this message is a little bit late processed
	if (!m_bConnected)
		return;

	// close the socket
	m_pAgent->DisconnectAgent();
	m_bTryingToConnect = FALSE;
	m_bConnected = FALSE;
}

void CBasisDoc::OnViewNographics() 
{
	m_bNoGraphics = !m_bNoGraphics;
	m_pBasisView->SetNoGraphics(m_bNoGraphics);
	m_pPerceptionView->SetNoGraphics(m_bNoGraphics);
}

void CBasisDoc::OnUpdateConnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected || m_bTryingToConnect)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CBasisDoc::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CBasisDoc::OnUpdateViewNographics(CCmdUI* pCmdUI) 
{
	if (m_bNoGraphics)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}
