// TPerception.cpp: implementation of the TPerception class.
//
//////////////////////////////////////////////////////////////////////

#include "OS.h"
#include "stdafx.h"
#include "Basis.h"
#include "TPerceptionView.h"
#include "TParseString.h"
#include "TViewingAgent.h"
#include "TPerception.h"
#include "TScreenParams.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TPerception::TPerception(TViewingAgent* pAgent)
{
	m_pAgent = pAgent;
	CreatePerception();
}

TPerception::TPerception(TViewingAgent* pAgent, TPerceptionView* pView)
{
	m_pAgent = pAgent;
	m_pView = pView;
	m_pView->SetPerception(this);
	CreatePerception();
}

void TPerception::CreatePerception()
{
	int	i;
	double	dFlagX[NO_OF_FLAGS] = FLAG_POSITIONS_X;
	double	dFlagY[NO_OF_FLAGS] = FLAG_POSITIONS_Y;

	// create the needed objects
	// players
	m_pThisPlayer = new TPlayer();
	m_pThisPlayer->SetPlayer(0, 0.0, 0, 0, 0, 0, -1, 0);
	for (i = 0; i < MAX_PLAYERS; i++)
		m_pVisiblePlayer[i] = new TPlayer();
	
	// goals
	m_pGoal[0] = new TSoccerGoal(-52.5, 0.0);
	m_pGoal[1] = new TSoccerGoal(52.5, 0.0);

	// flags
	for (i = 0; i < NO_OF_FLAGS; i++)
		m_pFlag[i] = new TFlag(dFlagX[i], dFlagY[i]);

	// lines
	for (i = 0; i < 5; i++)
		m_pLine[i] = new TLine();

	// ball
	m_pBall = new TBall();

	m_iNoOfPlayers = 0;
	m_iPlayMode = BEFORE_KICK_OFF;
	m_bViewLines = TRUE;
	m_iViewQuality = 1;
	m_iViewAngle = 90;
}

TPerception::~TPerception()
{
	int	i;

	// delete visible objects
	delete m_pThisPlayer;
	for (i = 0; i < MAX_PLAYERS; i++)
		delete m_pVisiblePlayer[i];

	// delete the flags
	for (i = 0; i < NO_OF_FLAGS; i++)
		delete m_pFlag[i];
	
	// delete the lines
	for (i = 0; i < 5; i++)
		delete m_pLine[i];

	delete m_pGoal[0];
	delete m_pGoal[1];
	delete m_pBall;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::DrawPerception(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
Returnval: (void)
Description: draws the last received perception
-----------------------------------------------------------------------------*/
void TPerception::DrawPerception(CDC* pDC, TScreenParams *tScreen)
{
	int		i;

	if (tScreen->IsTopView())
	{
		// draw the visible area
		m_pThisPlayer->DrawVisibleArea(pDC, tScreen, m_iViewAngle);

		// draw the goals
		m_pGoal[0]->DrawObject(pDC, tScreen);
		m_pGoal[1]->DrawObject(pDC, tScreen);

		// and the flags
		for (i = 0; i < NO_OF_FLAGS; i++)
			m_pFlag[i]->DrawObject(pDC, tScreen);

		// draw the directly perceived line tags
		for (i = 0; i < 5; i++)
			m_pLine[i]->DrawObject(pDC, tScreen);

		// draw the visible players
		for (i = 0; i < m_iNoOfPlayers; i++)
			m_pVisiblePlayer[i]->DrawObject(pDC, tScreen);

		// draw this player
		if (tScreen->IsTopView())
			m_pThisPlayer->DrawObject(pDC, tScreen);

		// draw the ball
		m_pBall->DrawObject(pDC, tScreen);
	}
	else
	{
		// draw from the players perspective
		// sort the objects depending on their distance
		CObList	drawQueue;
		
		Insert(drawQueue, m_pGoal[0]);
		Insert(drawQueue, m_pGoal[1]);

		// and the flags
		for (i = 0; i < NO_OF_FLAGS; i++)
			Insert(drawQueue, m_pFlag[i]);

		// draw the directly perceived line tags
		for (i = 0; i < 5; i++)
			Insert(drawQueue, m_pLine[i]);

		// draw the visible players
		for (i = 0; i < m_iNoOfPlayers; i++)
			Insert(drawQueue, m_pVisiblePlayer[i]);

		// draw the ball
		Insert(drawQueue, m_pBall);

		// draw the visible area
		m_pThisPlayer->DrawVisibleArea(pDC, tScreen, m_iViewAngle);

		// and draw them
		POSITION pos = drawQueue.GetHeadPosition();
		while (pos != NULL)
			((TVisibleObject*) drawQueue.GetNext(pos))->DrawObject(pDC, tScreen);
	}
}

/*-----------------------------------------------------------------------------
Function: void TPerception::Insert(CObList &list, TVisibleObject* obj)
Date    : 12.02.98
-------------------------------------------------------------------------------
Parameter: reference to the draw list
			  pointer to the object, that has to be inserted
Returnval: (void)
Description: inserts sorted, by distance of the object, descending
-----------------------------------------------------------------------------*/
void TPerception::Insert(CObList &list, TVisibleObject* obj)
{
	TVisibleObject*	compare;
	
	// only valid objects are drawn
	if (!obj->IsValid())
		return;

	POSITION oldPos;
	POSITION	pos = list.GetHeadPosition();
	while(pos != NULL)
	{
		oldPos = pos;
		compare = (TVisibleObject*) list.GetNext(pos);
		if (obj->GetDistance() >= compare->GetDistance())
		{
			list.InsertBefore(oldPos, obj);
			return;
		}
	}

	// it is the nearest object so far
	list.AddTail(obj);
}

/*-----------------------------------------------------------------------------
Function: int	TPerception::SwitchViewAngle(void)
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: the new view angle
Description: switches the internal representation of the viewing angle from 
				 180 degrees to 90 degrees to 45 degrees
-----------------------------------------------------------------------------*/
int	TPerception::SwitchViewAngle()
{ 
	if (m_iViewAngle == 180)
		m_iViewAngle = 90;
	else if (m_iViewAngle == 90)
		m_iViewAngle = 45;
	else
		m_iViewAngle = 180;
	
	return m_iViewAngle;
}	

/*-----------------------------------------------------------------------------
Function: void	TPerception::SwitchViewQuality(void)
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: switches the view quality from high to low and back to high
-----------------------------------------------------------------------------*/
int	TPerception::SwitchViewQuality()
{
	if (m_iViewQuality == 0)
		m_iViewQuality = 1;
	else
		m_iViewQuality = 0;

	return m_iViewQuality;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::NewMessage(LPCTSTR str)
Date    : 05.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the message
Returnval: (void)
Description: calls the views function for new message,which stores it and calls 
				 ParseMessage
-----------------------------------------------------------------------------*/
void TPerception::NewMessage(LPCTSTR str)
{
	m_pView->NewMessage(str);
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseMessage(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: string containing the perception (max 1024 char)
Returnval: TRUE, if no parsing errors occured
Description: distinguishes between see and hear messages and calls the appropriate
				 specialized parsing functions
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseMessage(LPCTSTR str)
{
	switch(str[1])
	{
		case 'h':
			// something to hear
			return ParseHear(str);
		
		case 's':
			// something to see or sense body
			if (str[3] == 'e')
				// a see message
				return ParseSee(str);
			else
				// a sense body information
				return ParseSenseBody(str);

		case 'i':
			// message at init
			return ParseInit(str);

		case 'r':
			// message at reconnect
			return ParseInit(str);
	}
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseSee(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: reference of the string to parse
			  reference to the current position
Returnval: TRUE, if no parsing errors occured
Description: parses a see perception
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSee(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	strObject;
	
	// delete previously seen players
	InvalidateAll();

	// get the time
	msg.SetPosition(5);
	m_iSeeTime = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	m_pThisPlayer->SetTime(m_iSeeTime);

	// get the visible objects
	while (msg.ParseBracket(strObject))
	{
		if (!ParseObject(strObject))
		{
			#ifdef _DEBUG
				CString error;
				error.Format(" : error parsing object info at position %d !\n", strObject.GetPosition());
				afxDump << error << strObject;
			#endif
		}
	};

	// get the absoulte localization of the player
	Localize();

	// TRUE, although may be not all objects could be parsed
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseHear(LPCTSTR str)
Date    : 10.11.97
-------------------------------------------------------------------------------
Parameter: reference to the current position in the string which is 0
Returnval: TRUE, if no parse error occured
Description: parses a hear perception
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseHear(LPCTSTR str)
{
	TParseString	msg(str);
	CString			tmp;
	int				mode = 0;
	char*				strPlayMode[] = PLAYMODE_STRINGS;

	// get the time
	msg.SetPosition(5);
	m_iHearTime = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;

	// get the source
	switch (msg.GetAt(msg.GetPosition()))
	{
		case 'r':
			// it is from the referee
			if (!msg.ParseString(tmp))
				return FALSE;
			// get the message
			if (!msg.ParseString(tmp))
				return FALSE;
			
			while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(strPlayMode[mode])))
				mode++;

			if (mode < NO_OF_PLAYMODES)
				m_iPlayMode = mode;
			break;
		
		case 's':
			// it has been sent by myself
			break;
		
		default:
			// it is a say message
			break;
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseSenseBody(LPCTSTR str)
Date    : 27.01.98
-------------------------------------------------------------------------------
Parameter: string to parse
Returnval: FALSE, if an error occured
Description: parses a sense body information (sense_body TIME ((view_mode QUALITY
				 WIDTH) (stamina STAMINA EFFORT) (speed AMOUNT) (kick COUNT) (dash COUNT)
				 (turn COUNT) (say COUNT))
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseSenseBody(LPCTSTR str)
{
	TParseString	msg(str);
	CString			temp;

	// read the time
	msg.SetPosition(12);
	m_iSenseTime = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;

	// read the view mode
	msg.ParseString(temp);
	msg.ParseString(temp);
	if (msg.HasErrorOccured())
		return FALSE;
	if (temp.Compare("low") == 0)
		m_iViewQuality = 0;
	else
		m_iViewQuality = 1;
	msg.ParseString(temp);
	if (msg.HasErrorOccured())
		return FALSE;
	if (temp.Compare("narrow") == 0)
		m_iViewAngle = 45;
	else if (temp.Compare("wide") == 0)
		m_iViewAngle = 180;
	else
		m_iViewAngle = 90;

	// read the stamina
	msg.ParseString(temp);
	m_iStamina = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	m_iEffort = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;

	// read the speed
	msg.ParseString(temp);
	m_iSpeed = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;

	// read the counts
	msg.ParseString(temp);
	m_iNoOfKick = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	
	msg.ParseString(temp);
	m_iNoOfDash = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	
	msg.ParseString(temp);
	m_iNoOfTurn = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	
	msg.ParseString(temp);
	m_iNoOfSay = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	
	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseInit(LPCTSTR str)
Date    : 29.01.98
-------------------------------------------------------------------------------
Parameter: string containing the init message (init side number playmode)
Returnval: FALSE, if an error occured
Description: parses the init message to get the side and players number
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseInit(LPCTSTR str)
{
	TParseString	msg(str);
	CString			tmp;
	int				number;
	int				mode;
	char*				strPlayMode[] = PLAYMODE_STRINGS;

	// get the side
	if ((msg.GetLength() > 6) && (msg.GetAt(6) == 'r'))
		m_pAgent->SetSide(RIGHT_TO_LEFT);
	else
		m_pAgent->SetSide(LEFT_TO_RIGHT);

	// get the number of the player
	msg.SetPosition(8);
	number = msg.ParseInteger();
	if (msg.HasErrorOccured())
		return FALSE;
	m_pAgent->SetNumber(number);

	// get the playmode
	msg.ParseString(tmp);
	mode = 0;
	while ((mode < NO_OF_PLAYMODES) && (tmp.Compare(strPlayMode[mode])))
		mode++;

	if (mode < NO_OF_PLAYMODES)
		m_iPlayMode = mode;

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::ParseObject(LPCTSTR str)
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: reference to the current position in the string, which must be the 
			  opening bracket of the objinfo
Returnval: TRUE, if no parse error occured
Description: parses an object and stores the information
-----------------------------------------------------------------------------*/
BOOL TPerception::ParseObject(LPCTSTR str)
{
	TParseString	msg(str);
	TParseString	name;
	CString		teamname;
	int			team;
	int			iNum;
	double		dist;
	int			dir;
	int			distchng;
	int			dirchng;
	int			faceDir;
	int			index;
	int			type;
	char			tag1;
	char			tag2;
	int			flagNum;
	int			iFlag;
	TFlag*		pFlag;
	TLine*		pLine;

	// read the objects name bracket
	if (!msg.ParseBracket(name))
		return FALSE;
	
	// read the distance, direction and distchngection (the last may be not existing)
	if (m_iViewQuality == 0)
	{
		// we have got low view quality, so no distances
		dist = NOT_EXISTING;
		dir = msg.ParseInteger();
		if (msg.HasErrorOccured())
			return FALSE;
	}
	else
	{
		// we have got high view quality
		dist = msg.ParseDouble();
		if (msg.HasErrorOccured())
			return FALSE;
		dir = msg.ParseInteger();
		if (msg.HasErrorOccured())
			return FALSE;
	}
	dirchng = NOT_EXISTING;
	distchng = msg.ParseInteger();
	if (msg.HasErrorOccured())
		distchng = NOT_EXISTING;
	else 
	{	
		dirchng = msg.ParseInteger();
		if (msg.HasErrorOccured())
			dirchng = NOT_EXISTING;
		else 
		{	
			faceDir = msg.ParseInteger();
			if (msg.HasErrorOccured())
				faceDir = NOT_EXISTING;
		}
	}

	// switch on the kind of object
	switch (name.GetAt(0))
	{
		// it is a player
		case 'P':
		case 'p':
			name.SetPosition(7);
			// read the teamname
			if (name.ParseString(teamname))
			{
				// ist it my team ?
				if (teamname.Compare(_T(MY_TEAM)) == 0)
					team = 0;
				else
					team = 1;

				// read player number
				iNum = name.ParseInteger();
				if (msg.HasErrorOccured())
					iNum = NOT_EXISTING;
			}
			else
			{
				team = NOT_EXISTING;
				iNum = NOT_EXISTING;
			} 

			m_pVisiblePlayer[m_iNoOfPlayers]->SetPlayer(m_iSeeTime, dist, dir, distchng
																		, dirchng, faceDir, team, iNum);
			m_iNoOfPlayers++;
			break;


		case 'G':
		case 'g':
			// it is a goal
			if ((name.GetLength() > 5) && (name.GetAt(5) == 'l'))
				index = 0;
			else
				index = 1;
			
			// store the values
			m_pGoal[index]->SetDistance(dist);
			m_pGoal[index]->SetDirection(dir);
			m_pGoal[index]->SetTime(m_iSeeTime);
						
			break;


		case 'B':
		case 'b':
			// it is the ball
			m_pBall->SetDistance(dist);
			m_pBall->SetDirection(dir);
			m_pBall->SetTime(m_iSeeTime);
			break;

		case 'F':
			// do not treat flags by now, that are behind me
			break;

		case 'f':
			// it is a flag
			type = FIELD_FLAG;
			if (name.GetLength() < 6)
				return FALSE;
			tag1 = name.GetAt(5);
			if (name.GetLength() < 8)
			{
				// it must be the center flag
				if (tag1 != 'c')
					return FALSE;
				iFlag = CENTER_MARK;
			}
			else
			{
				tag2 = name.GetAt(7);
				switch (tag1)
				{	
					case 'c':
						// it is a center flag 
						if (tag2 == 't')
							iFlag = CENTER_TOP;
						else
							iFlag = CENTER_BOTTOM;
						break;

					case 'g':
						// it is a goal flag
						tag1 = tag2;
						if (name.GetLength() < 10)
							return FALSE;
						tag2 = name.GetAt(9);
						if (tag1 == 'l')
						{
							if (tag2 == 't')
								iFlag = GOAL_POST_L;
							else
								iFlag = GOAL_POST_L + 1;
						}
						else
						{
							if (tag2 == 't')
								iFlag = GOAL_POST_R;
							else
								iFlag = GOAL_POST_R + 1;
						}
						break;

					case 'p':
						// it is a flag of the penalty area
						type = PENALTY_AREA_FLAG;
						tag1 = tag2;
						if (name.GetLength() < 10)
							return FALSE;
						tag2 = name.GetAt(9);
						if (tag1 == 'l')
						{
							if (tag2 == 't')
								iFlag = FLAG_PENALTY_L;
							else if (tag2 == 'c')
								iFlag = FLAG_PENALTY_L + 1;
							else
								iFlag = FLAG_PENALTY_L + 2;
						}
						else
						{
							if (tag2 == 't')
								iFlag = FLAG_PENALTY_R;
							else if (tag2 == 'c')
								iFlag = FLAG_PENALTY_R + 1;
							else
								iFlag = FLAG_PENALTY_R + 2;
						}
						
						break;

					case 'l':
						// it is one of the left flags of the field
						if (tag2 == '0')
							// it is the left center flag
							iFlag = FLAG_OUTSIDE_LC;
						else
						{
							name.SetPosition(9);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								// it is an old lt or lb flag
								return TRUE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 2)
								return FALSE;
							if (tag2 == 't')
								// it is a left top outside flag
								iFlag = FLAG_OUTSIDE_LT + flagNum;
							else
								// it is a left bottom outside flag
								iFlag = FLAG_OUTSIDE_LB + flagNum;
						}
						break;

					case 'r':
						// it is one of the right flags of the field
						if (tag2 == '0')
							// it is the right center flag
							iFlag = FLAG_OUTSIDE_RC;
						else
						{
							name.SetPosition(9);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								// it is an old rt or rb flag
								return TRUE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 2)
								return FALSE;
							if (tag2 == 't')
								// it is a right top outside flag
								iFlag = FLAG_OUTSIDE_RT + flagNum;
							else
								// it is a right bottom outside flag
								iFlag = FLAG_OUTSIDE_RB + flagNum;
						}
						break;

					case 't':
						// it is one of the top flags of the field
						if (tag2 == '0')
							// it is the top center flag
							iFlag = FLAG_OUTSIDE_TC;
						else
						{
							name.SetPosition(9);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								return FALSE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 4)
								return FALSE;
							if (tag2 == 'l')
								// it is a top left outside flag
								iFlag = FLAG_OUTSIDE_TL + flagNum;
							else
								// it is a top right outside flag
								iFlag = FLAG_OUTSIDE_TR + flagNum;
						}
						break;

					case 'b':
						// it is one of the bottom flags of the field
						if (tag2 == '0')
							// it is the bottom center flag
							iFlag = FLAG_OUTSIDE_BC;
						else
						{
							name.SetPosition(9);
							flagNum = name.ParseInteger();
							if (name.HasErrorOccured())
								return FALSE;
							flagNum /= 10;
							flagNum--;
							if ((flagNum < 0) || flagNum > 4)
								return FALSE;
							if (tag2 == 'l')
								// it is a bottom left outside flag
								iFlag = FLAG_OUTSIDE_BL + flagNum;
							else
								// it is a bottom right outside flag
								iFlag = FLAG_OUTSIDE_BR + flagNum;
						}
						break;

					default:
						iFlag = TRASH_FLAG;
				}
			}

			pFlag = m_pFlag[iFlag];
			pFlag->SetDistance(dist);
			pFlag->SetDirection(dir);
			pFlag->SetTime(m_iSeeTime);
			pFlag->SetFlag(type, tag1, tag2);

			break;

		case 'l':
			// it is a line
			if (name.GetLength() < 6)
				break;
			tag1 = name.GetAt(5);
			switch (tag1)
			{
				case 'l':
					pLine = m_pLine[0];
					break;

				case 'r':
					pLine = m_pLine[1];
					break;

				case 't':
					pLine = m_pLine[2];
					break;

				case 'b':
					pLine = m_pLine[3];
					break;
			}

			pLine->SetDistance(dist);
			pLine->SetDirection(dir);
			pLine->SetTime(m_iSeeTime);
			break;
	}

	return TRUE;
}

/*-----------------------------------------------------------------------------
Function: void TPerception::InvalidateAll()
Date    : 17.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: invalidates all visible objects. Invalidate means, they are not drawn
				 if they are not perceived in this cycle
-----------------------------------------------------------------------------*/
void TPerception::InvalidateAll()
{
	int	i;

	// ThisPlayer does not get invalidated, because it is always drawn
	// Invalidate Players
	m_iNoOfPlayers = 0;
	
	// and other objects
	for (i = 0; i < NO_OF_FLAGS; i++)
		m_pFlag[i]->Invalidate();

	for (i = 0; i < 5; i++)
		m_pLine[i]->Invalidate();

	m_pGoal[0]->Invalidate();
	m_pGoal[1]->Invalidate();
	m_pBall->Invalidate();
}

/*-----------------------------------------------------------------------------
Function: BOOL TPerception::Localize()
Date    : 16.02.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: TRUE, if the absolute position of the agent could be calculated
Description: calculates the absolute position of the agent. Two flags are 
				 necessary (with this algorithm) to locate the position. It is at one
				 of the two intersection points of the circles surounding the flags
				 in the distance of the player. Which intersection is decided by the 
				 relative angle the player sees the flags.
				 To calculate the intersection points, first the inbtersection point P3 of 
				 the line from flag1 to flag2 and the line from intersection point 1
				 and 2 is calculated. 
				 
-----------------------------------------------------------------------------*/
BOOL TPerception::Localize()
{
	int		i;
	int		iFlag1 = -1;
	int		iFlag2 = -1;
	int		iMaxAngle = -1000;
	int		iDeltaAngle;
	int		iFlag1Direction;
	double	r1, r2;				// the distance of the player to the flags 
	double	dist, dist2;		// the distance (square) of the two flags
	double	a;						// distance from one flag to intersection point P3
	double	h;						// distance from P3 to the intersection Points P1 and P2 of the two circles
	double	x, x1, x2, x3;
	double	y, y1, y2, y3;
	double	ratio;
	int		iAbsDirection;
	int		beta;

	// search for two visible flags with maximal difference of the view angle
	// this makes things more acurate
	for (i = 0; i < NO_OF_FLAGS; i++)
	{
		if (m_pFlag[i]->IsValid())
		{
			if (iFlag1 == -1)
			{
				// found the first visible flag
				iFlag1 = i;
				iFlag1Direction = m_pFlag[iFlag1]->GetDirection();
			}
			else
			{
				// found another visible flag
				iDeltaAngle = iFlag1Direction - m_pFlag[i]->GetDirection();
				if (iDeltaAngle < 0)
					iDeltaAngle = -iDeltaAngle;
				if (iDeltaAngle > iMaxAngle)
				{
					// this flag has a bigger angle to the first flag
					iFlag2 = i;
					iMaxAngle = iDeltaAngle;
				}
			}
		}
	}

	if (iFlag2 == -1)
		// there are not 2 visible flags
		return FALSE;

	// do the calculations
	r1 = m_pFlag[iFlag1]->GetDistance();
	r2 = m_pFlag[iFlag2]->GetDistance();

	x1 = m_pFlag[iFlag1]->GetAbsX();
	x2 = m_pFlag[iFlag2]->GetAbsX();

	y1 = m_pFlag[iFlag1]->GetAbsY();
	y2 = m_pFlag[iFlag2]->GetAbsY();

	// calculate the square distance of the two flags
	dist2 = (x2 - x1)*(x2 - x1) + (y2 - y1)*(y2 - y1);	
	dist = sqrt(dist2);
	if (dist > r1 + r2)
	{
		// the circles would not intersect (because of inaccuracy)
		dist = r1 + r2;
		dist2 = dist * dist;
	}
	else if ((r1 > r2) && (dist + r2 < r1))
	{
		// the circles would not intersect (inaccuracy)
		dist = r1 - r2;
		dist2 = dist * dist;
	}
	else if ((r2 > r1) && (dist + r1 < r2))
	{
		// the circles would not intersect
		dist = r2 - r1;
		dist2 = dist * dist;
	}

	r1 *= r1;
	r2 *= r2;
	
	// a = (r1^2 - r2^2 + d^2 ) / (2 d) 
	a = (r1 - r2 + dist2) / (2.0 * dist);
	// h^2 = r1^2 - a^2
	h = r1 - a*a;
	if (h < 0.0)
		h = 0.0;
	else
		h = sqrt(h);

	// P3 = P1 + a ( P2 - P1 ) / d
	x3 = x1 + a * (x2 - x1) / dist;
	y3 = y1 + a * (y2 - y1) / dist;

	// two circles intersect usually in 2 points. Find out which one to select
	if (m_pFlag[iFlag1]->GetDirection() > m_pFlag[iFlag2]->GetDirection())
	{
		// result x = x3 + h ( y2 - y1 ) / d 
		x = x3 + h * (y2 - y1) / dist;
		y = y3 - h * (x2 - x1) / dist;
	}
	else
	{
		x = x3 - h * (y2 - y1) / dist;
		y = y3 + h * (x2 - x1) / dist;
	}

	m_pThisPlayer->SetAbsX(x);
	m_pThisPlayer->SetAbsY(y);

/*
	#ifdef _DEBUG
		CString trace;
		trace.Format("Flag1: %d (%3.1f / %3.1f)  %4.2f   %4.1f / %4.1f  %4.1f %4.1f  (%4.1f / %4.1f)\n"
						, iFlag1, x1, y1, r1, x3, y3, a, h, x, y);
		afxDump << trace;
		trace.Format("Flag2: %d (%3.1f / %3.1f)  %4.2f\n", iFlag2, x2, y2, r2);
		afxDump << trace;
	#endif
*/

	// calculate the absolute direction
	r1 = m_pFlag[iFlag1]->GetDistance();
	ratio = (y1 - y) / r1;
	beta = (int) (asin(ratio) * 180.0 / PI);
	if (x > x1)
		iAbsDirection = 180 - beta - iFlag1Direction;
	else
		iAbsDirection = beta - m_pFlag[iFlag1]->GetDirection();

	if (iAbsDirection > 180)
		iAbsDirection -= 360;

	m_pThisPlayer->SetAbsDirection(iAbsDirection);

	return TRUE;
}


/*-----------------------------------------------------------------------------
Function: Functions returning properties of the world
Date    : 24.02.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: property
Description: 
-----------------------------------------------------------------------------*/
int TPerception::GetGoalDirection()
{
	int	index = 0;

	// which one is the correct goal
	if (m_pAgent->GetSide() == LEFT_TO_RIGHT)
		index = 1;

	// if the player can see the goal, then return visible direction
	if (m_pGoal[index]->IsValid())
		return m_pGoal[index]->GetDirection(); 
	else
		// the goal is not visible, so use the absolute position of the agent
		return m_pThisPlayer->GetAngleTo(m_pGoal[index]);
}