// TPerception.h: interface for the TPerception class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


#include "TVisibleObject.h"

#define NO_OF_FLAGS		50
#define CENTER_MARK		0
#define CENTER_TOP		1
#define CENTER_BOTTOM	2
#define GOAL_POST_L		3
#define GOAL_POST_R		5
#define FLAG_OUTSIDE_TL	7
#define FLAG_OUTSIDE_TC	12
#define FLAG_OUTSIDE_TR	13
#define FLAG_OUTSIDE_BL	18
#define FLAG_OUTSIDE_BC	23
#define FLAG_OUTSIDE_BR	24
#define FLAG_OUTSIDE_LT	29
#define FLAG_OUTSIDE_LC	32
#define FLAG_OUTSIDE_LB	33
#define FLAG_OUTSIDE_RT	36
#define FLAG_OUTSIDE_RC	39
#define FLAG_OUTSIDE_RB	40
#define FLAG_PENALTY_L	43
#define FLAG_PENALTY_R	46
#define TRASH_FLAG		49

#define FLAG_POSITIONS_X { 0.0, 0.0, 0.0, -52.5, -52.5, 52.5, 52.5\
				, -10.0, -20.0, -30.0, -40.0, -50.0, 0.0\
				, 10.0, 20.0, 30.0, 40.0, 50.0\
				, -10.0, -20.0, -30.0, -40.0, -50.0, 0.0\
				, 10.0, 20.0, 30.0, 40.0, 50.0\
				, -57.5, -57.5, -57.5, -57.5\
				, -57.5, -57.5, -57.5\
				, 57.5, 57.5, 57.5, 57.5\
				, 57.5, 57.5, 57.5\
				, -36.0, -36.0, -36.0, 36.0, 36.0, 36.0, 0.0 }

#define FLAG_POSITIONS_Y { 0.0, - 34.0, 34.0, -7.32, 7.32, -7.32, +7.32\
				, -39.0, -39.0, -39.0, -39.0, -39.0, -39.0\
				, -39.0, -39.0, -39.0, -39.0, -39.0\
				, 39.0, 39.0, 39.0, 39.0, 39.0, 39.0\
				, 39.0, 39.0, 39.0, 39.0, 39.0\
				, -10.0, -20.0, -30.0, 0.0\
				, 10.0, 20.0, 30.0\
				, -10.0, -20.0, -30.0, 0.0\
				, 10.0, 20.0, 30.0\
				, -20.16, 0.0, 20.16, -20.16, 0.0, 20.16, 0.0 }


class TScreenParams;
class TPerceptionView;
class TViewingAgent;

/*-----------------------------------------------------------------------------
Class   : TPerception
Date    : 07.11.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: contains the methods for parsing the perceptions
-----------------------------------------------------------------------------*/
class TPerception : public CObject  
{

	public:
		TPerception(TViewingAgent* pAgent);
		TPerception(TViewingAgent* pAgent, TPerceptionView* pView);
		virtual ~TPerception();
		
		void	CreatePerception();
		void	NewMessage(LPCTSTR msg);
		BOOL	ParseMessage(LPCTSTR msg);
		void	DrawPerception(CDC* pDC, TScreenParams *tScreen);

		// Attribute functions
		void	SetPlayMode(int mode) { m_iPlayMode = mode; };
		int	GetPlayMode(void) const { return m_iPlayMode; };
		void	SetViewLines(BOOL view) { m_bViewLines = view; };
		BOOL	GetViewLines(void) const { return m_bViewLines; };
		
		// proposition functions 
		int		GetBallDirection() const { return m_pBall->GetDirection(); };
		double	GetBallDistance() const { return m_pBall->GetDistance(); };
		int		GetGoalDirection();
		double	GetGoalDistance() const { return m_pGoal[0]->GetDistance(); };
		BOOL		GetSeeBall() const { if (m_iSeeTime == m_pBall->GetTime()) return TRUE; else return FALSE; };

		void	SetViewParams(int angle, int quality) { m_iViewAngle = angle; m_iViewQuality = quality; };
		int	SwitchViewAngle(void); 
		int	SwitchViewQuality(void);
		int	GetViewQuality(void) const { return m_iViewQuality; };
		int	GetViewAngle(void) const { return m_iViewAngle; };
		TViewingAgent*	GetAgent() { return m_pAgent; };

	private:

		BOOL	ParseSee(LPCTSTR str);
		BOOL	ParseHear(LPCTSTR str);
		BOOL	ParseSenseBody(LPCTSTR str);
		BOOL	ParseInit(LPCTSTR str);
		BOOL	ParseObject(LPCTSTR str);
		void	InvalidateAll(void);
		void	Insert(CObList &list, TVisibleObject* obj);
		BOOL	Localize(void);

		int		m_iSeeTime;				// time received from the last perception
		int		m_iHearTime;
		int		m_iSenseTime;			// time received from the last sense_body

		int		m_iPlayMode;				// mode the game is in (e.g. before_kickoff)
		int		m_iNoOfPlayers;			// number of currently visible players
		TPlayer*	m_pVisiblePlayer[MAX_PLAYERS];	// max 22 players
		TPlayer* m_pThisPlayer;

//		TPlayer*		m_pPlayer1[11];	// own team
//		TPlayer*		m_pPlayer2[11];	// other team
		TSoccerGoal* m_pGoal[2];		// 0 is the own goal
		TBall*		m_pBall;			// the ball
		TFlag*		m_pFlag[NO_OF_FLAGS];		// flags of the corners and midline
		TLine*		m_pLine[5];

		BOOL		m_bViewLines;
		
		// sense body information
		int		m_iViewQuality;
		int		m_iViewAngle;
		int		m_iStamina;
		int		m_iEffort;
		int		m_iSpeed;
		int		m_iNoOfKick;
		int		m_iNoOfDash;
		int		m_iNoOfTurn;
		int		m_iNoOfSay;

		TPerceptionView*	m_pView;
		TViewingAgent*		m_pAgent;
};

#endif // !defined(AFX_TPERCEPTION_H__DA8AA141_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
