// TPerceptionView.cpp : implementation file
//

#include "OS.h"
#include "stdafx.h"
#include "Basis.h"
#include "TScreenParams.h"
#include "TPerception.h"
#include "TViewingAgent.h"
#include "TPerceptionView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView

IMPLEMENT_DYNCREATE(TPerceptionView, CView)

TPerceptionView::TPerceptionView()
{
	m_pScreen = new TScreenParams();
	m_iNoOfCycles = 0;
	m_iCurrentCycle = 0;
	m_bRunning = FALSE;
	m_bChanged = FALSE;
	m_iTimerID = 0;
}

TPerceptionView::~TPerceptionView()
{
	// stop the timer
	if (m_bRunning)
		OnLogPerceptStop();

	delete m_pScreen;
}


BEGIN_MESSAGE_MAP(TPerceptionView, CView)
	//{{AFX_MSG_MAP(TPerceptionView)
	ON_WM_CHAR()
	ON_COMMAND(ID_ZOOM_PLUS, OnZoomPlus)
	ON_COMMAND(ID_VIEW_PLAYERS_VIEW, OnViewPlayersView)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PLAYERS_VIEW, OnUpdateViewPlayersView)
	ON_COMMAND(ID_ZOOM_MINUS, OnZoomMinus)
	ON_COMMAND(ID_VIEW_ANGLE, OnViewAngle)
	ON_COMMAND(ID_VIEW_QUALITY, OnViewQuality)
	ON_UPDATE_COMMAND_UI(ID_VIEW_QUALITY, OnUpdateViewQuality)
	ON_COMMAND(ID_LOG_PERCEPT_BACK, OnLogPerceptBack)
	ON_COMMAND(ID_LOG_PERCEPT_GOTO, OnLogPerceptGoto)
	ON_COMMAND(ID_LOG_PERCEPT_ONE_STEP, OnLogPerceptOneStep)
	ON_COMMAND(ID_LOG_PERCEPT_PLAY, OnLogPerceptPlay)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_BACK, OnUpdateLogPerceptBack)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_GOTO, OnUpdateLogPerceptGoto)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_ONE_STEP, OnUpdateLogPerceptOneStep)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_PLAY, OnUpdateLogPerceptPlay)
	ON_COMMAND(ID_FILE_SAVE_PERCEPTION, OnFileSavePerception)
	ON_COMMAND(ID_FILE_OPEN_PERCEPTION, OnFileOpenPerception)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN_PERCEPTION, OnUpdateFileOpenPerception)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_PERCEPTION, OnUpdateFileSavePerception)
	ON_COMMAND(ID_SAVE_AS_PERCEPTION, OnFileSaveAsPerception)
	ON_UPDATE_COMMAND_UI(ID_SAVE_AS_PERCEPTION, OnUpdateFileSaveAsPerception)
	ON_COMMAND(ID_LOG_PERCEPT_STOP, OnLogPerceptStop)
	ON_UPDATE_COMMAND_UI(ID_LOG_PERCEPT_STOP, OnUpdateLogPerceptStop)
	ON_WM_TIMER()
	ON_COMMAND(ID_COMMANDS, OnCommands)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView drawing

void TPerceptionView::OnDraw(CDC* pDC)
{
	RedrawView();
}

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView diagnostics

#ifdef _DEBUG
void TPerceptionView::AssertValid() const
{
	CView::AssertValid();
}

void TPerceptionView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}


#endif //_DEBUG

void TPerceptionView::NewMessage(LPCTSTR str)
{
	// check upper limit
	if (m_iNoOfCycles >= 12000)
		return;

	// only store see messages
	if ((str[1] == 's') && (str[3] == 'e'))
	{
		// save string
		m_Msg[m_iNoOfCycles] = str;
		m_iCurrentCycle = m_iNoOfCycles;
		m_iNoOfCycles++;
		m_bChanged = TRUE;

		RedrawView();
	}
	else
		m_pPerception->ParseMessage(str);
}

void TPerceptionView::RedrawView()
{
	CClientDC		pDC(this);
	CDC				dcMem;
	CBitmap			myBitmap;
	CRect				windowRect;
	int				sizeX, sizeY;

	if (m_pPerception->GetAgent() == NULL)
		return;

	// first call the parse function of perception, to draw the right things
	m_pPerception->ParseMessage(m_Msg[m_iCurrentCycle]);
	
	// parsing has to be done in any case, even when not drawing
	if (m_bNoGraphics)
		return;

	// prepare the memory device context
	dcMem.CreateCompatibleDC(&pDC);
	
	// get Device Coordinates
	pDC.GetWindow()->GetClientRect(&windowRect);
	windowRect.NormalizeRect();
	sizeX = windowRect.Width();
	sizeY = windowRect.Height();
	myBitmap.CreateCompatibleBitmap(&pDC, sizeX, sizeY);
	// set the size of the bitmap, to be read by the drawing functions
	myBitmap.SetBitmapDimension(sizeX, sizeY);
	
	ASSERT(myBitmap.m_hObject != NULL);

	CBitmap* pOldBitmap = dcMem.SelectObject(&myBitmap);

	// erase the memory device context
	dcMem.FillSolidRect(0, 0, sizeX, sizeY, COLOR_FIELD);

	// draw the things
	if (m_pScreen->IsTopView())
		m_pScreen->SetMid(sizeX / 2, sizeY - 30);
	else
		m_pScreen->SetMid(sizeX / 2, sizeY / 2);
	m_pPerception->DrawPerception(&dcMem, m_pScreen);

	// copy the memory bitmap into the window
	pDC.BitBlt(0, 0, sizeX, sizeY, &dcMem, 0, 0, SRCCOPY);

	// remove the created bitmap
	dcMem.SelectObject(pOldBitmap);
	dcMem.DeleteDC();
	myBitmap.DeleteObject();
}



void TPerceptionView::LoadFromArchive(CArchive &ar)
{
	int		i;

	// write the messages to the archive
	ar >> m_iNoOfCycles;
	for (i = 0; i < m_iNoOfCycles; i++)
		ar >> m_Msg[i];
	m_iCurrentCycle = 0;
	m_bChanged = FALSE;
}

void TPerceptionView::SaveToArchive(CArchive &ar)
{
	int		i;

	// write the messages to the archive
	ar << m_iNoOfCycles;
	for (i = 0; i < m_iNoOfCycles; i++)
		ar << m_Msg[i];
	m_bChanged = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// TPerceptionView message handlers
void TPerceptionView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// switch on the command received
	switch (nChar)
	{
		case ' ':
			m_pPerception->GetAgent()->Kick(100, 0);
			break;
		case 'j':
			m_pPerception->GetAgent()->Turn(-15);
			break;
		case 'l':
			m_pPerception->GetAgent()->Turn(15);
			break;
		case 'i':
				m_pPerception->GetAgent()->Dash(100);
			break;
		case 'k':
				m_pPerception->GetAgent()->Dash(-30);
			break;

		case VK_RETURN:
			m_pPerception->GetAgent()->Move(-5, 0);
			break;

		case 's':
			m_pPerception->GetAgent()->SenseBody();			
			break;

		default:
			CView::OnChar(nChar, nRepCnt, nFlags);
	}
}

BOOL TPerceptionView::PreCreateWindow(CREATESTRUCT& cs) 
{
	BOOL ret = CView::PreCreateWindow(cs);
	
	cs.style = AFX_WS_DEFAULT_VIEW | ES_NOHIDESEL;

	// set the initial parameters for displaying the perceptions
	// that is midx, midy, scale
	m_pScreen->SetScale(6.0);
	return ret;
}


void TPerceptionView::OnFileOpenPerception() 
{
	CFile		file;
	CString	message;

	// ask for the file to open
	CFileDialog fileDlg(TRUE, "*.le1", NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR
							  , "Perception Files (*.le1)|*.le1|All Files (*.*)|*.*", this);
	if (fileDlg.DoModal() != IDOK)
		return;
	m_strFileName = fileDlg.GetPathName();

	// open the file and archive
	if(!file.Open(m_strFileName, CFile::modeRead))
		return;
	
	CArchive	archive(&file, CArchive::load);

	// and load it
	LoadFromArchive(archive);
	archive.Close();
	file.Close();
	return;
}

void TPerceptionView::OnFileSaveAsPerception() 
{
	// ask for the filename
	CFileDialog fileDlg(FALSE, "*.le1", m_strFileName, OFN_HIDEREADONLY | OFN_NOCHANGEDIR
							  , "Perception Files (*.le1)|*.le1|All Files (*.*)|*.*|", this);
	if (fileDlg.DoModal() != IDOK)
		return;
	m_strFileName = fileDlg.GetPathName();
	
	if (m_strFileName.GetLength() > 0)
		OnFileSavePerception();
}

void TPerceptionView::OnFileSavePerception() 
{
	if (m_strFileName.GetLength() == 0)
		// the file has no name
		OnFileSaveAsPerception();
	else
	{
		CFile		file;
		CString	message;

		// open the file and archive
		if(!file.Open(m_strFileName, CFile::modeCreate | CFile::modeWrite))
		{
			message.Format("Unable to open file %s for writing", m_strFileName);
			AfxMessageBox(message);
			return;
		}
		
		CArchive	archive(&file, CArchive::store);

		// and save it
		SaveToArchive(archive);
		archive.Close();
		file.Close();
	}
}

void TPerceptionView::OnLogPerceptBack() 
{
	m_iCurrentCycle = 0;
	RedrawView();
}

void TPerceptionView::OnLogPerceptGoto() 
{
	// TODO: Add your command handler code here
	
}

void TPerceptionView::OnLogPerceptOneStep() 
{
	m_iCurrentCycle++;
	if (m_iCurrentCycle >= m_iNoOfCycles)
		m_iCurrentCycle = m_iNoOfCycles - 1;
	RedrawView();
}

void TPerceptionView::OnLogPerceptPlay() 
{
	if (m_bRunning)
		return;

	m_bRunning = TRUE;
	m_iTimerID = 1;
	m_iTimerID = SetTimer(m_iTimerID, 150, NULL);
}


void TPerceptionView::OnLogPerceptStop() 
{
	if (!m_bRunning)
		return;

	KillTimer(m_iTimerID);
	m_bRunning = FALSE;
	m_iTimerID = 0;
}

void TPerceptionView::OnTimer(UINT nIDEvent) 
{
	m_iCurrentCycle++;
	if (m_iCurrentCycle >= m_iNoOfCycles)
	{
		m_iCurrentCycle = m_iNoOfCycles - 1;
		OnLogPerceptStop();
	}
	else
		RedrawView();

	CView::OnTimer(nIDEvent);
}

void TPerceptionView::OnViewAngle() 
{
	m_pPerception->GetAgent()->SwitchViewAngle();
	RedrawView();
}

void TPerceptionView::OnViewQuality() 
{
	m_pPerception->GetAgent()->SwitchViewQuality();
	RedrawView();
}

void TPerceptionView::OnZoomMinus() 
{
	// zoom out of the players perceptions
	m_pScreen->Scale(0.75);
}

void TPerceptionView::OnZoomPlus() 
{
	// zoom in into the players perceptions
	m_pScreen->Scale(1.333);
}

void TPerceptionView::OnViewPlayersView() 
{
	// switch from top view to 2 1/2 D view
	m_pScreen->SetTopView(!m_pScreen->IsTopView());
	RedrawView();
}

void TPerceptionView::OnUpdateViewPlayersView(CCmdUI* pCmdUI) 
{
	if (m_pScreen->IsTopView())
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}


void TPerceptionView::OnUpdateViewQuality(CCmdUI* pCmdUI) 
{
	if (m_pPerception->GetViewQuality() == 1)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}


void TPerceptionView::OnUpdateLogPerceptBack(CCmdUI* pCmdUI) 
{
	if (m_iCurrentCycle > 0)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptGoto(CCmdUI* pCmdUI) 
{
	if (m_iNoOfCycles > 0)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptOneStep(CCmdUI* pCmdUI) 
{
	if ((m_iNoOfCycles > 0) && (m_iCurrentCycle < m_iNoOfCycles - 1))
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptPlay(CCmdUI* pCmdUI) 
{
	if ((m_iNoOfCycles > 0) && !m_bRunning)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);

	if (m_bRunning)
		pCmdUI->SetRadio(TRUE);
	else
		pCmdUI->SetRadio(FALSE);
}

void TPerceptionView::OnUpdateLogPerceptStop(CCmdUI* pCmdUI) 
{
	if (m_bRunning)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateFileOpenPerception(CCmdUI* pCmdUI) 
{
}

void TPerceptionView::OnUpdateFileSavePerception(CCmdUI* pCmdUI) 
{
	if (m_bChanged)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void TPerceptionView::OnUpdateFileSaveAsPerception(CCmdUI* pCmdUI) 
{
}



void TPerceptionView::OnCommands() 
{
	// shows the command keys
	AfxMessageBox("Keys to control the agent: \n\n i,k : dash\n j,l : turn\n space: kick");
}
