// TViewingAgent.cpp: implementation of the TViewingAgent class.
//
//////////////////////////////////////////////////////////////////////

#include "OS.h"
#include "stdafx.h"
#include "Basis.h"
#include "BasisView.h"
#include "TScreenParams.h"
#include	"TPerception.h"
#include "TViewingAgent.h"
#include "DSetupDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TViewingAgent::TViewingAgent()
{
	m_pPerception = new TPerception(this);
}

TViewingAgent::TViewingAgent(CBasisView* pBView, TPerceptionView* pPView)
{
	m_pView = pBView;
	m_pPerception = new TPerception(this, pPView);
}

TViewingAgent::~TViewingAgent()
{
	delete m_pPerception;
}

/*-----------------------------------------------------------------------------
Function: BOOL TViewingAgent::Connect()
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: tries to connect with the standard connection parameters. if not 
				 successfull, asks the user for new parameters
-----------------------------------------------------------------------------*/
BOOL TViewingAgent::Connect()
{
	DSetupDlg Dialog;

	// ask the user for the name of the team and the server
	Dialog.m_strHandle = _T("Team1");
	Dialog.m_strServer = _T("ultra2");
	Dialog.m_nChannel = 6000;

	// user can repeat with a different address
	while(TRUE)
	{
		// try to connect
		if (ConnectAgent(Dialog.m_strHandle, Dialog.m_strServer, Dialog.m_nChannel))
			return TRUE;
		
		if (Dialog.DoModal() != IDOK)
			return FALSE;
	}
	// change the view parameters of the server to the current view
//	ChangeView(m_pPerception->GetViewAngle()
//					, m_pPerception->GetViewQuality());
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::SetViewParams(int angle, int quality)
Date    : 20.01.98
-------------------------------------------------------------------------------
Parameter: view angle (one of 180, 90 or 45)
			  view quality (0 for low, 1 for high)
Returnval: void
Description: sets the internal representation of the viewing angle and quality
-----------------------------------------------------------------------------*/
void TViewingAgent::OnChangedView(int angle, int quality)
{ 
	m_pPerception->SetViewParams(angle, quality); 
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::SwitchViewQuality()
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: switches the view quality from high to low
-----------------------------------------------------------------------------*/
void TViewingAgent::SwitchViewQuality()
{
	int iQuality = m_pPerception->SwitchViewQuality();
	int iAngle = m_pPerception->GetViewAngle();
	ChangeView(iAngle, iQuality);
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::SwitchViewAngle()
Date    : 19.12.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: switches from a view of 180 to 90 to 45 degrees
-----------------------------------------------------------------------------*/
void TViewingAgent::SwitchViewAngle()
{
	int iAngle = m_pPerception->SwitchViewAngle();
	int iQuality = m_pPerception->GetViewQuality();
	ChangeView(iAngle, iQuality);
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::OnReceivedMessage(LPCTSTR message)
Date    : 16.12.97
-------------------------------------------------------------------------------
Parameter: message that has been received from the server
Returnval: void
Description: calls the TPerception ParseMessage method to parse the message
				 calls the views display message function
-----------------------------------------------------------------------------*/
void TViewingAgent::OnReceivedMessage(LPCTSTR message)
{
	// parse the message
	m_pPerception->NewMessage(message);

	// display the message in the message view
	m_pView->Message(message);
}

/*-----------------------------------------------------------------------------
Function: void TViewingAgent::OnSentMessage(LPCTSTR message)
Date    : 17.12.97
-------------------------------------------------------------------------------
Parameter: message that has been sent to the server
Returnval: (void)
Description: displays the message in the message view
-----------------------------------------------------------------------------*/
void TViewingAgent::OnSentMessage(LPCTSTR message)
{
	m_pView->Message(message);
}

/*-----------------------------------------------------------------------------
Function: higher level actions
Date    : 24.02.98
-------------------------------------------------------------------------------
Parameter: 
Returnval: (void)
Description: actions with a higher level of abstraction
-----------------------------------------------------------------------------*/
void TViewingAgent::ComplexDash(int iPower, int iDir)
{
	// just working on
}

void TViewingAgent::ComplexKick(int iPower, int iDirection)
{
	// avoids to kick onto the agent itself
	int	angle = iDirection - m_pPerception->GetBallDirection();

	if (angle < -180)
		angle += 360;
	else if (angle > 180)
		angle -= 360;
	
	if (angle > 165)
		Kick(20, 90);	
	else if (angle < -165)
		Kick(20, -90);	
	else
		Kick(iPower, iDirection);	
}
