// TVisibleObject.cpp: implementation of the TVisibleObject class.
//
//////////////////////////////////////////////////////////////////////
#include "OS.h"
#include "stdafx.h"
#include "Basis.h"
//#include "Vector.h"
#include "TVisibleObject.h"
#include "TScreenParams.h"

#include	<math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


/*-----------------------------------------------------------------------------
Function: TVisibleObject::TVisibleObject()
Date    : 07.11.97
-------------------------------------------------------------------------------
Parameter: none
Description: constructor
-----------------------------------------------------------------------------*/
TVisibleObject::TVisibleObject()
{
	m_iTime = 0;
	m_dDistance = 0.0;
	m_iDirection = 0;
	m_bValid = FALSE;
	m_iDistChng = 0;
	m_iDirChng = 0;
	m_dAbsX = 0.0;
	m_dAbsY = 0.0;
	m_iAbsDirection = 0;
}

TVisibleObject::~TVisibleObject()
{
}

/*-----------------------------------------------------------------------------
Function: void GetCoordinates(int &x, int &y, TScreenParams *tScreen)
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: reference to the coordinates to return
Returnval: (void)
Description: calculates screen coordinates from distance and direction
-----------------------------------------------------------------------------*/
void TVisibleObject::SetCoordinates(TScreenParams *tScreen)
{
	// rad is x * pi / 180
	if (tScreen->IsTopView())
	{
		// the perception is drawn from a top view
		m_iX = tScreen->GetMitteX() + (int) (tScreen->GetScale() * m_dDistance 
												 * cos(((double) m_iDirection - 90.0) * PI / 180.0)); 	
		m_iY = tScreen->GetMitteY() + (int) (tScreen->GetScale() * m_dDistance 
												 * sin(((double) m_iDirection - 90.0) * PI / 180.0)); 	
	}
	else
	{
		double x;
		double y;
		// it is drawn as seen by the player
		x = m_dDistance * cos(((double) m_iDirection - 90.0) * PI / 180.0); 	
		y = - m_dDistance * sin(((double) m_iDirection - 90.0) * PI / 180.0); 	

		// m_iX = x * distance of viewplane / y
		if (y > 0.01)
			m_iX = tScreen->GetMitteX() + (int) (x * VIEWPLANE_DIST * tScreen->GetScale()/ y);
		else
			m_iX = 1000;

		// m_iY = distance to view plane * height of eye / distance of object 
		if (m_dDistance > 0.01)
			m_iY = tScreen->GetMitteY() + (int) (VIEWPLANE_DIST * tScreen->GetScale()
															* EYE_HEIGHT / m_dDistance);
		else
			m_iY  = 1000;
	}
}


/*-----------------------------------------------------------------------------
Function: void TPlayer::SetPlayer(int iTime, double dist, int dir, int distchng
							 , int dirchng, int team, int num)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: sets the players member variables
-----------------------------------------------------------------------------*/
void TPlayer::SetPlayer(int iTime, double dist, int dir, int distchng
							 , int dirchng, int faceDir, int team, int num)
{
	// sets the parameters of a player
	m_iTime = iTime;
	m_dDistance = dist;
	m_iDirection = dir;
	m_bValid = TRUE;
	m_iDistChng = distchng;
	m_iDirChng = dirchng;
	m_iFaceDir = faceDir;
	m_iTeam = team;
	m_iNum = num;

	// need to set speed ???
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
Returnval: (void)
Description: draws the player as a circle (with the number in if existing)
-----------------------------------------------------------------------------*/
void TPlayer::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// what team is the player
	if (m_iTeam == 0)
		cCol = COLOR_MYTEAM;
	else if (m_iTeam == 1)
		cCol = COLOR_OTHERTEAM;
	else if (m_iTeam == -1)
		cCol = COLOR_AGENT;
	else
		cCol = COLOR_ANY_OBJECT;

// cCol = RGB(GetRValue(cCol) - color, GetGValue(cCol) - color, GetBValue(cCol) - color);

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	if (tScreen->IsTopView())
		size = PLAYERSIZE;
	else if (m_dDistance > EYE_HEIGHT)
		size = (int) (PLAYERSIZE  * SCALE_DIST / m_dDistance);
	else
		size = (int) (PLAYERSIZE  * SCALE_DIST / EYE_HEIGHT);

	// draw a Ellipse at the players position
	pDC->Ellipse(m_iX - size, m_iY - size, m_iX + size, m_iY + size);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TPlayer::DrawVisibleArea(CDC* pDC, TScreenParams *tScreen)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
			  size of the angle the player is viewing (45, 90, 180)
Returnval: (void)
Description: draws the visible area depending on the view angle
-----------------------------------------------------------------------------*/
void TPlayer::DrawVisibleArea(CDC* pDC, TScreenParams *tScreen, int iViewAngle)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	
	if (tScreen->IsTopView())
	{
		CPoint	point[4];
		// view seen from the top
		// color of the visible area
		SetCoordinates(tScreen);
	
		// draw the polygone
		cCol = COLOR_VISIBLE_AREA;
		brush.CreateSolidBrush(cCol);
		oldBrush = pDC->SelectObject(&brush);

		// draw the visible area
		point[0].x = m_iX;
		point[0].y = m_iY;
		if (iViewAngle == 90)
		{
			// 90 degrees view, it is a equaly sided triangle
			point[1].x = m_iX - m_iY;
			point[1].y = 0;
			point[2].x = m_iX + m_iY;
			point[2].y = 0;
			pDC->Polygon(point, 3);
		}
		else if (iViewAngle == 180)
		{
			// 180 degrees, it is a rectangle
			point[0].x = 0;
			point[0].y = 0;
			point[1].x = tScreen->GetMitteX() * 2;
			point[1].y = 0;
			point[2].x = point[1].x;
			point[2].y = m_iY;
			point[3].x = 0;
			point[3].y = m_iY;
			pDC->Polygon(point, 4);
		}
		else
		{
			// 45 degrees view
			point[1].x = m_iX - m_iY / 2;
			point[1].y = 0;
			point[2].x = m_iX + m_iY / 2;
			point[2].y = 0;
			pDC->Polygon(point, 3);
		}
	}
	else
	{
		// view seen from players view
		// draw the rectangle
		cCol = COLOR_SKY;
		brush.CreateSolidBrush(cCol);
		oldBrush = pDC->SelectObject(&brush);

		// draw the sky
		pDC->Rectangle(0, 0, tScreen->GetMitteX() * 2, tScreen->GetMitteY());
	}

	// draw the time and coordinates of this player
	int		nPointSize;
	CFont		font;
	CFont*	oldFont;
	CRect		field;
	COLORREF	oldTextColor;
	COLORREF	oldBkColor;
	int		sizeX;
	CString	time;

	nPointSize = (int) (30.0 * tScreen->GetScale());
	if (!font.CreatePointFont(nPointSize, _T("Times New Roman"), pDC))
		font.CreateStockObject(SYSTEM_FONT);
	oldFont = pDC->SelectObject(&font);

	// calculate the rectangle of the text
	sizeX = (int) tScreen->GetScale() * nPointSize / 5;
	oldTextColor = pDC->SetTextColor(0x00000000);
	oldBkColor = pDC->SetBkColor(COLOR_FIELD);

	// draw time
	field.SetRect(5, 5, sizeX, nPointSize / 5);
	time.Format("%5d", m_iTime);
	pDC->DrawText(time, field, DT_SINGLELINE);

	// draw coordinates
	field.SetRect(tScreen->GetMitteX() * 2 - sizeX, 5, tScreen->GetMitteX() * 2 - 5, nPointSize / 5);
	time.Format("(%4.1f %4.1f) %3d", m_dAbsX, m_dAbsY, m_iAbsDirection);
	pDC->DrawText(time, field, DT_SINGLELINE | DT_RIGHT);

	pDC->SetTextColor(oldTextColor);
	pDC->SetBkColor(oldBkColor);
	pDC->SelectObject(oldFont);
	font.DeleteObject();

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: int TPlayer::GetAngleTo(TVisibleObject* obj)
Date    : 24.02.98
-------------------------------------------------------------------------------
Parameter: pointer to the object, to which the rel angle is calculated
Returnval: relative angle of the passed object
Description: calculates the rel angle. If the object is ahead it is 0 left is - right is +
-----------------------------------------------------------------------------*/
int TPlayer::GetAngleTo(TVisibleObject* obj)
{
	double	dx, dy;
	double	ratio;
	int		beta;
	int		angle;

	dx = obj->GetAbsX() - m_dAbsX;
	dy = obj->GetAbsY() - m_dAbsY;
	if ((dx > D_ZERO_N) && (dx < D_ZERO_P))
	{
		// points are on same x coordinates
		if (dy > 0.0)
			angle = 90 - m_iAbsDirection;
		else
			angle = -90 - m_iAbsDirection;
	}
	else
	{
		ratio = dy / dx;
		beta = (int) (atan(ratio) * 180.0 / PI);
		if (dy > 0.0)
			angle = beta - m_iAbsDirection;
		else
			angle = -beta - m_iAbsDirection;
	}

	if (angle < -180)
		angle += 360;
	else if (angle > 180)
		angle -= 360;

	return angle;
}

/*-----------------------------------------------------------------------------
Function: void TBall::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the ball as a circle 
-----------------------------------------------------------------------------*/
void TBall::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the ball
	cCol = COLOR_BALL;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a Ellipse at the players position
	if (tScreen->IsTopView())
		size = BALLSIZE;
	else if (m_dDistance > EYE_HEIGHT)
		size = (int) (BALLSIZE  * SCALE_DIST / m_dDistance);
	else
		size = (int) (BALLSIZE  * SCALE_DIST / EYE_HEIGHT);

	pDC->Ellipse(m_iX - size, m_iY - size, m_iX + size, m_iY + size);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}


TSoccerGoal::TSoccerGoal(double x, double y)
{
	m_dAbsX = x;
	m_dAbsY = y;
}

/*-----------------------------------------------------------------------------
Function: void TSoccerGoal::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the goal as a black box
-----------------------------------------------------------------------------*/
void TSoccerGoal::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the ball
	cCol = COLOR_GOAL;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a rectangle
	if (tScreen->IsTopView())
		size = GOALSIZE;
	else if (m_dDistance > EYE_HEIGHT)
		size = (int) (GOALSIZE  * SCALE_DIST / m_dDistance);
	else
		size = (int) (GOALSIZE  * SCALE_DIST / EYE_HEIGHT);
	pDC->Rectangle(m_iX - size, m_iY - size, m_iX + size, m_iY + size);
	
	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: TFlag::TFlag(double x, double y)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: none
Description: a flag is by default a field flag
-----------------------------------------------------------------------------*/
TFlag::TFlag(double x, double y)
{
	m_dAbsX = x;
	m_dAbsY = y;
	m_iType = FIELD_FLAG;
}

/*-----------------------------------------------------------------------------
Function: _
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: _
Returnval: (void)
Description: _
-----------------------------------------------------------------------------*/
void TFlag::SetFlag(int type, char tag1, char tag2)
{
	m_iType = type;
	m_cTag1 = tag1;
	m_cTag2 = tag2;
}

/*-----------------------------------------------------------------------------
Function: void TFlag::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 13.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the flag as a triangle
-----------------------------------------------------------------------------*/
void TFlag::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	CPoint	point[3];
	int		size;

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the ball
	cCol = COLOR_FLAG;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a Ellipse at the players position
	if (tScreen->IsTopView())
		size = FLAG_SIZE;
	else if (m_dDistance > EYE_HEIGHT)
		size = (int) (FLAG_SIZE  * SCALE_DIST / m_dDistance);
	else
		size = (int) (FLAG_SIZE  * SCALE_DIST / EYE_HEIGHT);
	if (m_iType == FIELD_FLAG)
	{
		point[0].x = m_iX - size;
		point[0].y = m_iY - size;
		point[1].x = m_iX + size;
		point[1].y = m_iY;
		point[2].x = m_iX - size;
		point[2].y = m_iY + size;
	}
	else
	{
		point[0].x = m_iX + size;
		point[0].y = m_iY - size;
		point[1].x = m_iX - size;
		point[1].y = m_iY;
		point[2].x = m_iX + size;
		point[2].y = m_iY + size;
	}
	pDC->Polygon(point, 3);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void	TFlag::DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen)
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: pointer to the flag to which the line is drawn
			  pointer to the device context on which should be drawn
			  pointer to the screen parameters for scaling and translation
Returnval: (void)
Description: draws a line from this flag to the passed flag, if both flags are
				 seen from this point of view
-----------------------------------------------------------------------------*/
void	TFlag::DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen)
{
	CPen	pen;
	CPen* oldPen;
	COLORREF cCol = COLOR_LINE;

	// test if the two flags are valid
	if (!IsValid() || !pFlag->IsValid())
		return;

	// prepare the pen
	pen.CreatePen(PS_SOLID, 1, cCol);
	oldPen = pDC->SelectObject(&pen);

	// get the proper coordinates
	pDC->MoveTo(m_iX, m_iY);
	pDC->LineTo(pFlag->GetX(), pFlag->GetY());

	pDC->SelectObject(oldPen);
	pen.DeleteObject();
}

/*-----------------------------------------------------------------------------
Function: void TLine::DrawObject(CDC* pDC, TScreenParams *tScreen)
Date    : 18.12.97
-------------------------------------------------------------------------------
Parameter: pointer to the device context to draw into
			  reference to the sreen paramters (mid, scale)
Returnval: (void)
Description: draws the flag as a triangle
-----------------------------------------------------------------------------*/
void TLine::DrawObject(CDC* pDC, TScreenParams *tScreen)
{
	COLORREF cCol;
	CBrush	brush;
	CBrush	*oldBrush;
	CPoint	point[4];

	// draw only players having been percepted this cycle
	if (!IsValid())
		return;

	// color of the ball
	cCol = COLOR_LINE;

	// draw the polygone
	brush.CreateSolidBrush(cCol);
	oldBrush = pDC->SelectObject(&brush);
	SetCoordinates(tScreen);

	// draw a raute at the lines position
	point[0].x = m_iX;
	point[0].y = m_iY - FLAG_SIZE;
	point[1].x = m_iX + FLAG_SIZE;
	point[1].y = m_iY;
	point[2].x = m_iX;
	point[2].y = m_iY + FLAG_SIZE;
	point[3].x = m_iX - FLAG_SIZE;
	point[3].y = m_iY;
	pDC->Polygon(point, 4);

	pDC->SelectObject(oldBrush);
	brush.DeleteObject();
}
