// TVisibleObject.h: interface for the TVisibleObject class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define GOALSIZE			10
#define VIEWPLANE_DIST	30.0
#define EYE_HEIGHT		2.0
#define SCALE_DIST		50.0

//class TVector;
class TScreenParams;

/*-----------------------------------------------------------------------------
Class   : TVisibleObject
Date    : 11.11.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: class containing information of visible objects of an agent
-----------------------------------------------------------------------------*/
class TVisibleObject : public CObject  
{
	public:
		TVisibleObject();
		virtual ~TVisibleObject();

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen) {};
		void		SetCoordinates(TScreenParams *tScreen);

		// get and set functions
		void		SetTime(int time) { m_iTime = time; };
		void		SetDistance(double dist) { m_dDistance = dist; m_bValid = TRUE;};
		void		SetDirection(int dir) { m_iDirection = dir; };
		void		SetDistChng(int vdir) { m_iDistChng = vdir; };
		void		SetDirChng(int vdir) { m_iDistChng = vdir; };
		void		Invalidate(void) { m_bValid = FALSE; };
		void		SetAbsX(double x) { m_dAbsX = x; };
		void		SetAbsY(double y) { m_dAbsY = y; };
		void		SetAbsDirection(int dir) { m_iAbsDirection = dir; };

		int		GetTime(void) const { return m_iTime; };
		double	GetDistance(void) const { return m_dDistance; };
		int		GetDirection(void) const { return m_iDirection; };
		int		GetDistChng(void) const { return m_iDistChng; };
		int		GetDirChng(void) const { return m_iDirChng; };
		int		GetAge(int time) const { return (time - m_iTime); };
		BOOL		IsValid(void) const { return m_bValid; };
		int		GetX() const { return m_iX; };
		int		GetY() const { return m_iY; };
		double	GetAbsX() const { return m_dAbsX; };
		double	GetAbsY() const { return m_dAbsY; };
		int		GetAbsDirection() const { return m_iAbsDirection; };

	protected:

		int		m_iTime;			// time, when this perception has been made
		double	m_dDistance;	// rel. distance of the object to the agent
		int		m_iDirection;	// angle with respect to the agents position
		BOOL		m_bValid;		// true, if this object has been percepted this cycle
		int		m_iDistChng;	// change of distance
		int		m_iDirChng;		// change of direction

		int		m_iX;				// coordinates for displaying
		int		m_iY;

		double	m_dAbsX;			// absolute coordinates (of flags)
		double	m_dAbsY;
		int		m_iAbsDirection; // absoulte view direction
};

/*-----------------------------------------------------------------------------
Class   : TMovableObject
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TMoveableObject : public TVisibleObject
{
	public:
		TMoveableObject() {};
		virtual ~TMoveableObject() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen) {};

	protected:
//		TVector*	m_tSpeed;		// speedvector of the object
};

/*-----------------------------------------------------------------------------
Class   : TPlayer
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TMoveableObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TPlayer : public TMoveableObject
{
	public:
		TPlayer() {};
		virtual ~TPlayer() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
		void	DrawVisibleArea(CDC* pDC, TScreenParams *tScreen, int iViewAngle);

		// get and set functions
		void	SetPlayer(int iTime, double dist, int dir, int vdir
							 , int dirchng, int faceDir, int team, int num);
		void	SetTeam(int team) { m_iTeam = team; };
		void	SetNum(int num) { m_iNum = num; };

		int	GetTeam(void) const { return m_iTeam; };
		int	GetNum(void) const { return m_iNum; };

		int	GetAngleTo(TVisibleObject* obj);

	protected:
		int	m_iTeam;		// 0 if own team, 1 if other team
		int	m_iNum;		// number of player
		int	m_iFaceDir; // facing direction of the player
};

/*-----------------------------------------------------------------------------
Class   : TBall
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TMoveableObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TBall : public TMoveableObject
{
	public:
		TBall() {};
		virtual ~TBall() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
};

/*-----------------------------------------------------------------------------
Class   : TSoccerGoal
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TSoccerGoal : public TVisibleObject
{
	public:
		TSoccerGoal(double x, double y);
		virtual ~TSoccerGoal() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
};

/*-----------------------------------------------------------------------------
Class   : TFlag
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TFlag : public TVisibleObject
{
	public:
		TFlag(double x, double y);
		virtual ~TFlag() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
		void	DrawLine(TFlag* pFlag, CDC *pDC, TScreenParams *tScreen);
		void	SetFlag(int type, char tag1, char tag2);

	private:
		int		m_iType;		// penalty area or play area
		char		m_cTag1;		// char indicating which one
		char		m_cTag2;		// char indicating which one
};

/*-----------------------------------------------------------------------------
Class   : TLine
Date    : 13.11.97
-------------------------------------------------------------------------------
Base Class: TVisibleObject

Description: class containing information of movable visible objects of an agent
-----------------------------------------------------------------------------*/
class TLine : public TVisibleObject
{
	public:
		TLine() {};
		virtual ~TLine() {};

		virtual void DrawObject(CDC* pDC, TScreenParams *tScreen);
};

#endif // !defined(AFX_TVISIBLEOBJECT_H__DA8AA142_577A_11D1_B242_0000C09B5CBE__INCLUDED_)
