// SoccerCoachView.cpp : implementation of the CSoccerCoachView class
//

#include "stdafx.h"
#include "SoccerCoach.h"

#include "TCoachAgent.h"
#include "SoccerCoachDoc.h"
#include "SoccerCoachView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView

IMPLEMENT_DYNCREATE(CSoccerCoachView, CEditView)

BEGIN_MESSAGE_MAP(CSoccerCoachView, CEditView)
	//{{AFX_MSG_MAP(CSoccerCoachView)
	ON_COMMAND(ID_STARTCOACHING, OnStartcoaching)
	ON_UPDATE_COMMAND_UI(ID_STARTCOACHING, OnUpdateStartcoaching)
	ON_WM_TIMER()
	ON_COMMAND(ID_CONNECT, OnConnect)
	ON_UPDATE_COMMAND_UI(ID_CONNECT, OnUpdateConnect)
	ON_COMMAND(ID_DISCONNECT, OnDisconnect)
	ON_UPDATE_COMMAND_UI(ID_DISCONNECT, OnUpdateDisconnect)
	ON_COMMAND(IDC_PLAY_ON, OnPlayOn)
	ON_UPDATE_COMMAND_UI(IDC_PLAY_ON, OnUpdatePlayOn)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView construction/destruction

CSoccerCoachView::CSoccerCoachView()
{
	m_iTimerID = 0;
	m_bCoachRunning = FALSE;
	m_bConnected = FALSE;
}

CSoccerCoachView::~CSoccerCoachView()
{
}

BOOL CSoccerCoachView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CEditView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView drawing

void CSoccerCoachView::OnDraw(CDC* pDC)
{
	CSoccerCoachDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView printing

BOOL CSoccerCoachView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSoccerCoachView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSoccerCoachView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView diagnostics

#ifdef _DEBUG
void CSoccerCoachView::AssertValid() const
{
	CEditView::AssertValid();
}

void CSoccerCoachView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CSoccerCoachDoc* CSoccerCoachView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSoccerCoachDoc)));
	return (CSoccerCoachDoc*)m_pDocument;
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView operations

void CSoccerCoachView::Message(LPCTSTR lpszMessage)
{
	CString strTemp = lpszMessage;
	
	strTemp += _T("\r\n");
	int len = GetWindowTextLength();
	GetEditCtrl().SetSel(len,len);
	GetEditCtrl().ReplaceSel(strTemp);
}


/////////////////////////////////////////////////////////////////////////////
// CSoccerCoachView message handlers

void CSoccerCoachView::OnTimer(UINT nIDEvent) 
{
	// each 100 ms the coach agent does sth
	GetDocument()->m_pAgent->DoCoaching();		
	
	CEditView::OnTimer(nIDEvent);
}

void CSoccerCoachView::OnStartcoaching() 
{
	if (m_bCoachRunning)
	{
		// the coach is already running, so stop it
		KillTimer(m_iTimerID);
		m_iTimerID = 0;
		m_bCoachRunning = FALSE;
	}
	else
	{
		// start the timer
		m_iTimerID = 1;
		m_iTimerID = SetTimer(m_iTimerID, 1000, NULL);
		m_bCoachRunning = TRUE;
	}
}

void CSoccerCoachView::OnPlayOn() 
{
	GetDocument()->m_pAgent->SetPlayOn();			
}

void CSoccerCoachView::OnConnect() 
{
	// connect as coach
	GetDocument()->m_pAgent->Connect();	
	m_bConnected = TRUE;
}

void CSoccerCoachView::OnDisconnect() 
{
	// if this message is a little bit late processed
	if (!m_bConnected)
		return;

	// stop coaching
	if (m_bCoachRunning)
		OnStartcoaching();

	// close the socket
	GetDocument()->m_pAgent->DisconnectAgent();
	m_bConnected = FALSE;
}


void CSoccerCoachView::OnUpdateConnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

void CSoccerCoachView::OnUpdateDisconnect(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CSoccerCoachView::OnUpdateStartcoaching(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
	{
		pCmdUI->Enable(TRUE);
		if (m_bCoachRunning)
			pCmdUI->SetRadio(TRUE);
		else
			pCmdUI->SetRadio(FALSE);
	}
	else
	{
		pCmdUI->Enable(FALSE);
		pCmdUI->SetRadio(FALSE);
	}
}

void CSoccerCoachView::OnUpdatePlayOn(CCmdUI* pCmdUI) 
{
	if (m_bConnected)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

BOOL CSoccerCoachView::DestroyWindow() 
{
	if (m_iTimerID != 0)
		KillTimer(m_iTimerID);

	OnDisconnect();

	return CEditView::DestroyWindow();
}

