// BaseAgent.h: interface for the CBaseAgent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_)
#define AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Agentsock.h"


#define BEFORE_KICK_OFF		0
#define TIME_OVER				1
#define PLAY_ON				2
#define KICK_OFF_L			3
#define KICK_OFF_R			4
#define KICK_IN_L				5
#define KICK_IN_R				6
#define FREE_KICK_L			7
#define FREE_KICK_R			8 
#define CORNER_KICK_L		9
#define CORNER_KICK_R		10
#define GOAL_KICK_L			11
#define GOAL_KICK_R			12
#define GOAL_L					13 
#define GOAL_R					14 
#define DROP_BALL				15

#define NO_OF_PLAYMODES		16

#define HALF_TIME				21
#define TIME_UP				22
#define EXTEND					23
#define FOUL_L					50
#define FOUL_R					60


#define PLAYMODE_STRINGS {"before_kick_off",\
			"time_over",\
			"play_on",\
			"kick_off_l",\
			"kick_off_r",\
			"kick_in_l",\
			"kick_in_r",\
			"free_kick_l",\
			"free_kick_r",\
			"corner_kick_l",\
			"corner_kick_r",\
			"goal_kick_l",\
			"goal_kick_r",\
			"goal_l",\
			"goal_r",\
			"drop_ball"}


/*-----------------------------------------------------------------------------
Class   : CBaseAgent
Date    : 30.10.97
-------------------------------------------------------------------------------
Base Class: CObject

Description: base class for all agents
-----------------------------------------------------------------------------*/
class TBaseAgent : public CAgentSocket
{
	public:

		TBaseAgent();
		virtual ~TBaseAgent();
		BOOL	ConnectAgent(LPCTSTR lpszTeam, LPCTSTR lpszServer, UINT nPort);
		void	DisconnectAgent(void);

		// communication functions
		virtual void OnReceive(int nErrorCode);
		void SendMsg(CString& strText);
	
		// overridables
		virtual void OnCreatedSocket();
		virtual void OnSentMessage(LPCTSTR message) {};
		virtual void OnReceivedMessage(LPCTSTR message) {};
		
		virtual void OnErrorCreatingSocket() {};
		virtual void OnErrorSendingMessage() {};
		virtual void OnErrorReceivingMessage() {};
		
		// the agents actions
		void	Move(int x, int y);
		void	Turn(int iMoment);
		void	Dash(int iPower);
		void	Kick(int iPower, int iDirection);
		void	Say(CString csMessage);
		void	ChangeView(int iWidth, int iQuality);

		void  MoveRel(int x);	// moves relative to the current position

	protected:

		CString			m_strTeam;	// name of the team the player is in
	
		int				m_iPositionX;	// position of the agent x coordinate
		int				m_iPositionY;	// position of the agent y coordinate
		int				m_iDirection;	// view direction of the agent

		BOOL				m_bConnected;	// true, if the agent is currently connected

};

#endif // !defined(AFX_BASEAGENT_H__7FCEA561_5114_11D1_B242_0000C09B5CBE__INCLUDED_)
