// TCoachAgent.cpp: implementation of the TCoachAgent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SoccerCoach.h"
#include "SoccerCoachDoc.h"
#include "TCoachAgent.h"
#include "TParseString.h"
#include "DConnectionSetup.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TCoachAgent::TCoachAgent(CSoccerCoachDoc*	pDoc)
{
	m_bGoal = FALSE;
	m_bOutOfField = FALSE;
	m_pDoc = pDoc;
}

TCoachAgent::~TCoachAgent()
{
}


/*-----------------------------------------------------------------------------
Function: void TViewingAgent::Connect()
Date    : 14.11.97
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: tries to connect with the standard connection parameters. if not 
				 successfull, asks the user for new parameters
-----------------------------------------------------------------------------*/
void TCoachAgent::Connect()
{
	DConnectionSetup Dialog;

	// delete the current doc
	//OnNewDocument();

	// ask the user for the name of the team and the server
	Dialog.m_strHandle = _T("Team1");
	Dialog.m_strServer = _T("ultra2");
	Dialog.m_nChannel = 6001;

	// user can repeat with a different address
	while(TRUE)
	{
		// try to connect
		if (ConnectAgent(Dialog.m_strHandle, Dialog.m_strServer, Dialog.m_nChannel))
			return;
		
		if (Dialog.DoModal() != IDOK)
			return;
	}
}

void TCoachAgent::OnCreatedSocket()
{
}

/*-----------------------------------------------------------------------------
Function: void TCoachAgent::DoCoaching(void)
Date    : 12.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: main function to controll the players and the ball
-----------------------------------------------------------------------------*/
void TCoachAgent::DoCoaching(void)
{
	PlayMode		mode = PM_PlayOn;
	
	if (m_bGoal || m_bOutOfField)
	{
		// there has been a goal or the ball is outside
		CoachMove("(ball)", 0, 0, 0);
		ChangeMode(mode);
		m_bGoal = FALSE;
		m_bOutOfField = FALSE;
		m_pDoc->DisplayMessage("Moved the ball back to the origin");
	}

	// call for information for the next cycle
	CheckBall();
}

/*-----------------------------------------------------------------------------
Function: void TCoachAgent::SetPlayOn()
Date    : 20.01.98
-------------------------------------------------------------------------------
Parameter: none
Returnval: (void)
Description: sets the playmode of play on
-----------------------------------------------------------------------------*/
void TCoachAgent::SetPlayOn()
{
	PlayMode		mode = PM_PlayOn;
	
	ChangeMode(mode);
}


/*-----------------------------------------------------------------------------
Function: void TCoachAgent::OnReceivedMsg(LPCTSTR message)
Date    : 19.01.98
-------------------------------------------------------------------------------
Parameter: string containing the message received from the socket (server)
Returnval: (void)
Description: parses a received message
-----------------------------------------------------------------------------*/
void TCoachAgent::OnReceivedMessage(LPCTSTR message)
{
	TParseString	str(message);
	CString			temp;
	double			time;			

	// overread leading bracket
	str.m_iPosition = 1;
	str.ParseString(temp);
	if (temp.Compare("ok") != 0)
	{
		// it is an error or hear message

	}
	else
	{
		// it is a response
		str.ParseString(temp);
		if (temp.Compare("check_ball") == 0)
		{
			// it is a check ball information
			// get the time
			time = str.ParseDouble();
			str.ParseString(temp);
			if (temp.Compare("in_field") == 0)
			{
				// ball is in the field
				m_bGoal = FALSE;
				m_bOutOfField = FALSE;
			}
			else if (temp.Compare("out_of_field") == 0)
			{
				// ball is outside the field
				m_bGoal = FALSE;
				m_bOutOfField = TRUE;
			}
			else
			{
				// it has been a goal
				m_bGoal = TRUE;
				m_bOutOfField = TRUE;
			}
		}
		// all other ok informations are not relevant
	}
}

/*-----------------------------------------------------------------------------
Function: void TCoachAgent::CoachMove(LPCTSTR objname, int x, int y, int vdir)
Date    : 09.01.98
-------------------------------------------------------------------------------
Parameter: name of the object (ball or player) to move
			  coordinates to move the objects to
			  viewing direction of the object
Returnval: (void)
Description: a coach can move any object within the field
-----------------------------------------------------------------------------*/
void TCoachAgent::CoachMove(LPCTSTR objname, int x, int y, int vdir)
{
	CString	message;

	message.Format("(move %s %d %d %d)", objname, x, y, vdir);
	SendMsg(message);
}

/*-----------------------------------------------------------------------------
Function: void TCoachAgent::ChangeMode(int mode)
Date    : 12.01.98
-------------------------------------------------------------------------------
Parameter: play mode to change to
Returnval: (void)
Description: changes the playmode
-----------------------------------------------------------------------------*/
void TCoachAgent::ChangeMode(int mode)
{
	if (mode >= 0 && mode < NO_OF_PLAYMODES)
	{
		CString	message;
		char* playmode[16] = PLAYMODE_STRINGS;

		message.Format("(change_mode %s)", playmode[mode]);
		SendMsg(message);
	}
}

void TCoachAgent::CheckBall()
{
	SendMsg(CString("(check_ball)"));
}