/*
 *Header:
 *File: field.h (for C++)
 *Author: Noda Itsuki
 *Date: 1996/02/16
 *EndHeader:
 */

#include <stdio.h>

/*
 *===================================================================
 *Part: Stadium Class
 *===================================================================
 */
class Coach {
 public:
	Logical	assignedp ;
	Client	client ;
	Stadium *stadium ;
	int	socketfd ;
	int 	eye ;
	int	hear ;

	Coach(Stadium *std) { stadium = std; assignedp = False ; 
			      hear = FALSE ; } ;
	void parse_command(Stadium& std, char *command,
			unsigned long host,
			int port) ;
	void look(Stadium& std) ;
	void send_visual_info(void) ;
	void check_ball(Stadium& std) ;
	void udp_send(const char *msg) ;
} ;

class OnlineCoach : public Coach {
 public:
	Angle angle ;
	VisualInfo *vi ;
	int side ;

	OnlineCoach(Stadium *std) : Coach(std)
		{ angle = Deg2Rad(90) ; side = FALSE ;} ;
	void parse_command(Stadium& std, char *command,
			unsigned long host,
			int port) ;
	int scanField() ;
	void say(char *message) ;
} ;

class Field {
 public:
	PObject	goal_l ;
	PObject	goal_r ;
	PObject	line_l ;
	PObject	line_r ;
	PObject	line_t ;
	PObject	line_b ;

	void assign(Stadium *std) ;
} ;

class Weather {
 public:
	PVector	wind_vector ;
	Value	wind_rand ;

	void	init(Stadium *std) ;
} ;

class Stadium {
 public:
	Field	field ;
	Weather	weather ;
	dispinfo_t dinfo ;
	dispinfo_t minfo ;
	displist_t top ;

	MPObjectTable motable ;	/* Movable objects */
	PObjectTable votable ;	/* Visible objects */
	MPObject *ball ;
	PVector prev_ball_pos ;
	PVector offside_pos ;
	Player **player ;
//	TheNumber n_player ;
	TheNumber dest[MAX_PLAYER*2] ;
	Team *team_l ;
	Team *team_r ;
	TheNumber time ;
	TheNumber after_g_time ;
	TheNumber after_offside_time ;

	Player *last_touch ;
	Player *last_kicker ;
	Side kick_off_side ;
	PlayMode mode ;
	int caught_ball ;	/* whether goalie has catch ball or not */

	TheNumber startp ;	/* flag for physical simulator */

	/* server[1 socket] + players[MAX_PLAYER * 2] */
	int socketfd[MAX_PLAYER*2 + 1] ;
	int olcoachfd[3] ;

	 /* for commandline/configure file option */
	Value gwidth ;					/* goal width */
	Value inertia_moment ;			/* intertia moment for turn */
	Value psize ;					/* player size */
	Value pdecay ;					/* player decay */
	Value prand ;					/* player rand */
	Value pweight ;					/* player weight */
	Value pspeed_max ;				/* player speed max */
	Value stamina_max ;				/* player stamina max */
	Value stamina_inc ;				/* player stamina inc */
	Value recover_init ;			/* player recovery init */
	Value recover_dthr ;			/* player recovery decriment threshold */
	Value recover_min ;				/* player recovery min */
	Value recover_dec ;				/* player recovery decriment */
	Value effort_init ;				/* player dash effort init */
	Value effort_dthr ;				/* player dash effort decriment threshold */
	Value effort_min ;				/* player dash effrot min */
	Value effort_dec ;				/* player dash effort decriment */
	Value effort_ithr ;				/* player dash effort incriment threshold */
	Value effort_inc ;				/* player dash effort incriment */
	Value bsize ;					/* ball size */
	Value bdecay ;					/* ball decay */
	Value brand ;					/* ball rand */
	Value bweight ;					/* ball weight */
	Value bspeed_max ;				/* ball speed max */
	Value dprate ;					/* dash power rate */
	Value kprate ;					/* kick power rate */
	Value kmargin ;					/* kickable margin */
	Value ctlradius ;				/* control radius */
	Value ctlradius_width ;			/* (control radius) - (plyaer size) */
	Value maxp ;					/* max power */
	Value minp ;					/* min power */
	Value maxm ;					/* max moment */
	Value minm ;					/* min moment */
	Value maxnm ;					/* max neck moment */
	Value minnm ;					/* min neck moment */
	Value maxn ;					/* max neck angle */
	Value minn ;					/* min neck angle */
	Value visangle ;				/* visible angle */
	Value visdist ;					/* visible distance */
	Angle windir ;					/* wind direction */
	Value winforce ;				/* wind force */
	Value winang ;					/* wind angle for rand */
	Value winrand ;					/* wind force for force */
	Value kickable_area ;			/* kickable_area */
	Value catch_area_l ;			/* goalie catchable area length */
	Value catch_area_w ;			/* goalie catchable area width */
	Value catch_prob ;				/* goalie catchable possibility */
	Value ckmargin ;				/* corner kick margin */
	Value offside_area ;			/* offside active area size */
	int win_no ;					/* wind factor is none */
	int win_random ;				/* wind factor is random */
	int portnum ;					/* port number */
	int coach_pnum ;				/* coach port number */
	int olcoach_pnum ;				/* online coach port number */
	int say_cnt_max ;				/* max count of coach SAY */
	int SayCoachMsgSize ;				/* max length of coach SAY */
	int half_time ;					/* half time */
	int sim_st ;					/* simulator step interval msec */
	int send_st ;					/* udp send step interval msec */
	int recv_st ;					/* udp recv step intercal msec */
	int sb_step ;					/* sense_body interval step msec */
	int SayMsgSize ;				/* string size of say message */
	int hear_max ;					/* player hear_capacity_max */
	int hear_inc ;					/* player hear_capacity_inc */
	int hear_decay ;				/* player hear_capacity_decay */
	int cban_cycle ;				/* goalie catch ban cycle */
	int useoffside ;				/* flag for using off side rule */
	int kickoffoffside ;			/* flag for permit kick off offside */
	Value offside_kick_margin ;		/* offside kick margin */
	Value audio_dist ;				/* audio cut off distance */
	Value dist_qstep ;				/* quantize step of distance */
	Value land_qstep ;				/* quantize step of distance for landmark */
	int	verbose ;					/* flag for verbose mode */

	char logfile[MAX_FILE_LEN] ;	/* server log file name */
	char recfile[MAX_FILE_LEN] ;	/* log file name for recording */
	int  rec_ver ;					/* version for recording */
	int  rec_log ;					/* flag for log recording */
	int  rec_msg_mode ;				/* flag for whether to record msg_mode structures */
	int  send_log ;					/* flag for log sending */
	char replay[MAX_FILE_LEN] ;		/* log file name for replay */
	FILE *log_fp ;					/* file pointer for server log */
	FILE *rec_fp ;					/* file pointer for recording */

	int CoachMode ;				/* coach mode */
	int CwRMode ;					/* coach with referee mode */
	int old_hear ;					/* old format for hear command (coach) */
	Coach *coach ;
	OnlineCoach **olcoach ;
	int sv_st ;					/* online coach's look interval step */
	 
	void init(int argc, char **argv) ;
	void GetOption(int argc, char **argv) ;
	void MainLoop() ;
	void ReplayLoop() ;
	void check_io(int) ;
	void assign(void) ;
	MPObject* NewObject(
			const MPObjectType type,
			const Name name,
			const PVector pos,
			const Value size,
			const Value angle,
			const Value decay,
			const Value rand,
			const Value weight,
			const Value max_speed) ;
	void assignBall(void) ;

	Team *assocTeam(Name name) ;
	void initPlayer(void) ; 
	void initCoach(Stadium *std) ;
	void initOnlineCoach(Stadium *std) ;
	Player* newPlayer(const Name teamname, Value version, int goalie_flag, 
						unsigned long host, int port) ;
	OnlineCoach* newCoach(long unsigned int, int, char *) ;
	Player* get_client_player(unsigned long host, int port) ;
	Player* newClient(unsigned long host, int port, char* init_message) ;
	Player* reconnectClient(unsigned long host, int port, char* init_message) ;
	void byeClient(Player *bye_player) ;

	void referee_get_foul(double x, double y, int side) ;
	void referee_drop_ball(double x, double y, int side) ;
	void discard_player(int side, int unum) ;
	void goalie_catch(Player *goalie) ;
	int mark_offside_mark(Player *kicker) ;
	void offside_check(void) ;
	void offside(void) ;

	void step() ;
	char* now() ;

	void referee_step() ;
	void check_mode() ;
	void check_goal() ;
	void check_ball_in_pitch() ;
	void set_ball(PVector p,Side s, PVector vel=PVector(0.0,0.0)) ;
	void set_ball(PVector p,Side s,Angle ang, PVector vel=PVector(0.0,0.0)) ;
	void check_players_move() ;
	void check_player_after_offside() ;
	void check_goalie_while_catch(Player *goalie) ;
	void check_field_before_goalkick() ;
	void set_players() ;
	void recovery_players() ;
	void check_player_in_field() ;
	void reset_player_comm_flag() ;
	void announce_goal(Team& tm) ;

	void change_play_mode(PlayMode pm) ;

	int udp_init(int fd, int portnum) ;
	void udp_recv_message() ;
	void udp_send_message() ;
	void udp_send(const char *msg, Player &player,
			const Logical logp = TRUE) ;	
	void udp_send(const char *msg, unsigned long host, int port, int fd) ;
	void udp_send(const char *msg, unsigned long host, int port, int fd, int online) ;
	void udp_send(dispinfo_t *msg, unsigned long host, int port, int fd) ;
	void send_init_info(Player& player) ;
	void send_reconnect_info(Player& player) ;
	void send_audio_info(char *info, Player *id, int olcoach_side) ;
	void say(char *message) ;

	Logical coachp() { return coach->assignedp ; } ;
	void udp_recv_from_coach() ;
	void udp_recv_from_online_coach() ;
	void move_object(char *obj_name, PVector pos,
		PVector vel=PVector(0.0,0.0)) ;
	void move_object(char *obj_name, PVector pos,Angle ang,
		PVector vel=PVector(0.0,0.0));
	void check_collision(char *obj_name) ;
	void check_collision(MPObject *obj) ;
	Player* get_client_coach(unsigned long host, int port) ;
	Player* get_player_by_name(char *name) ;
	BallPosInfo check_ball_pos() ;
	Logical cross_gline(Side side) ;

	void OpenForWrite() ;
	void WriteLog(dispinfo_t *) ;
private:
	void ParsePlayerCommand(char* message, int iPlayer, unsigned long host, int port);
	bool ParseMonitorCommand(char* message, unsigned long host, int port);
} ;

/*
 *===================================================================
 *Part: ProtoTypes
 *===================================================================
 */

extern void _Start(Stadium& std) ;

/*
 *===================================================================
 *Part: for Netif.C
 *===================================================================
 */

extern void start_udp(Stadium *std) ;
