/* -*- Mode: C -*-
 *Header:
 *File: netif.C (for C++ & cc)
 *Author: Noda Itsuki
 *Date: 1996/02/14
 *EndHeader:
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef SYSV
#ifndef Solaris
#include <strings.h>
#endif
#endif
//#include <unistd.h>
#include <sys/types.h>
//#include <socket.h>
#include <winsock2.h>
#ifdef Solaris
#include <sys/filio.h>
#else
//#include <ioctl.h>
#endif
//#include <netinet/in.h>
//#include <arpa/inet.h>
//#include <netdb.h>
//#include <errno.h>
#include <math.h>
#include <fcntl.h>

#include <iostream.h>
//#include <strstream>
#include <strstrea.h>
#include "netif.h"
#include "param.h"
#include "types.h"
#include "utility.h"
#include "object.h"
#include "field.h"


void write_log(Stadium& std, Player& p, const char *message, int flag) ;
Value distance(PVector *pos1, PVector *pos2) ;


/*
 *===================================================================
 *Part: Initialize 
 *===================================================================
 */

int Stadium::udp_init(int fd, int portnum)
{
	struct sockaddr_in serv_addr, olcoach_addr, coach_addr ;
	unsigned long one ;
	int x;

	if (fd >= 0) {
		/* Open UDP socket. */

	if ((socketfd[fd] = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
		x= WSAGetLastError ();
		if (verbose)
			cerr << "Can't open socket." << endl ;
		return x ;
	}
		/* Set non-blocking. */
	one = 1 ;
	if(ioctlsocket(socketfd[fd], FIONBIO, &one) < 0 ){
		if (verbose)
			cerr << "Can't ioctl on socket" << endl ;
		return WSAGetLastError ();
	}

		/* Bind local address */
	bzero((char *) &serv_addr, sizeof(serv_addr)) ;
	serv_addr.sin_family		= AF_INET ;
	serv_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
	serv_addr.sin_port			= htons(portnum) ;

	if(bind(socketfd[fd], (struct sockaddr *) &serv_addr,
		sizeof(serv_addr)) < 0){
		if (verbose)
			cerr << "Can't bind local address" << endl ;
		return WSAGetLastError ();
	}

		if( fd != 0) return 0 ;

		/* Open olcoach socket. */
		if ((olcoachfd[0] = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
			if (verbose)
				cerr << "Can't open socket." << endl ;
			return WSAGetLastError () ;
		}
	
		/* Set non-blocking. */
		one = 1 ;
	if(ioctlsocket(olcoachfd[fd], FIONBIO, &one) < 0 ){
		if (verbose)
			cerr << "Can't ioctl on socket" << endl ;
		return WSAGetLastError ();
	}
	
		/* Bind local address */
		bzero((char *) &olcoach_addr, sizeof(serv_addr)) ;
		olcoach_addr.sin_family		= AF_INET ;
		olcoach_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
		olcoach_addr.sin_port		= htons(olcoach_pnum) ;
	
		if(bind(olcoachfd[0], (struct sockaddr *) &olcoach_addr,
			sizeof(serv_addr)) < 0){
			if (verbose)
				cerr << "Can't bind local address" << endl ;
			return WSAGetLastError() ;
		}

	if(CoachMode || CwRMode) {
		/* open coach socket */
		if((coach->socketfd = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
			if (verbose)
				cerr << "Can't open socket." << endl ;
			return WSAGetLastError ();
		}

		one = 1 ;
		if(ioctlsocket(coach->socketfd, FIONBIO, &one) < 0) {
			if (verbose)
				cerr << "Can't ioctl on socket." << endl ;
		}

		bzero((char *) &coach_addr, sizeof(coach_addr)) ;
		coach_addr.sin_family		= AF_INET ;
		coach_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
		coach_addr.sin_port			= htons(coach_pnum) ;

		if(bind(coach->socketfd, (struct sockaddr *) &coach_addr,
											sizeof(coach_addr)) < 0){
#ifdef DEBUG
			cerr << "Can't bind local address" << endl ;
#endif
			return WSAGetLastError ();
		}
	}
	}else {	/* online coach (fd < 0) */
		fd *= -1 ;
		/* Open UDP socket. */
		if ((olcoachfd[fd] = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
			if (verbose)
				cerr << "Can't open socket." << endl ;
			return WSAGetLastError ();
		}

		/* Set non-blocking. */
		one = 1 ;
		if(ioctlsocket(olcoachfd[fd], FIONBIO, &one) < 0 ){
			if (verbose)
				cerr << "Can't ioctl on socket" << endl ;
			return WSAGetLastError ();
		}

		/* Bind local address */
		bzero((char *) &olcoach_addr, sizeof(olcoach_addr)) ;
		olcoach_addr.sin_family		= AF_INET ;
		olcoach_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
		olcoach_addr.sin_port		= htons(portnum) ;

		if(bind(olcoachfd[fd], (struct sockaddr *) &olcoach_addr,
			sizeof(olcoach_addr)) < 0){
			if (verbose)
				cerr << "Can't bind local address" << endl ;
			return WSAGetLastError ();
		}

		olcoach[fd]->socketfd = olcoachfd[fd] ;
			
		return 0 ;
	}
	return 0;
}

/*
 *===================================================================
 *Part: Recv
 *===================================================================
 */
void Stadium::udp_recv_message()
{
	static char message[MaxMesg] ;

	int i, len, num ;
	int iMsgLength;
	int buf, sw ;

	/* randomize order */
	for(i = 0 ; i < MAX_PLAYER * 2; i++) 
	{
		sw = irand(MAX_PLAYER * 2) ;
		buf = dest[i] ;
		dest[i] = dest[sw] ;
		dest[sw] = buf ;
	}

	/* receive from socket for server(i=0) and players(i=1...) */
	for ( i = 0 ; i <= MAX_PLAYER * 2; i++) 
	{
		if (i != 0) 
		{
			num = dest[i-1] + 1 ;
			if (socketfd[num] == -1)
				continue ;
		}

		struct sockaddr_in cli_addr ;
		len = sizeof(cli_addr) ;

		/* read all messages of a player */ 
		iMsgLength = 1 ; 
		while (iMsgLength >= 0) 
		{
			if (i == 0)		/* server socket */
				iMsgLength = recvfrom(socketfd[0], message, MaxMesg, 0,
						      (struct sockaddr *) &cli_addr, &len) ;
			else			/* client socket */
				iMsgLength = recvfrom(socketfd[num], message, MaxMesg, 0,
		       			             (struct sockaddr *) &cli_addr, &len) ;

			/* treat errors */
			if (iMsgLength == -1 && WSAGetLastError() == WSAEWOULDBLOCK) 
				// no more messages of this player
				continue;
			else if(iMsgLength == -1 && WSAGetLastError() != WSAENOTSOCK)
			{
				if (verbose)
					cerr << "recvfrom error" << endl ;
				continue;
			}
			else if(iMsgLength == 0)
				continue ;

			unsigned long host = ntohl(cli_addr.sin_addr.s_addr) ;
			int port = ntohs(cli_addr.sin_port) ;

			/* chop newline */
			if(message[iMsgLength - 1] == '\n') 
				iMsgLength--;
			message[iMsgLength] = NULLCHAR ;

			/* soccermonitor commands */
			if (ParseMonitorCommand(message, host, port))
				continue;

			/* player commands */
			ParsePlayerCommand(message, i, host, port);
		}
	}
}


void Stadium::ParsePlayerCommand(char *message, int iPlayer, unsigned long host, int port)
{
	Player *p ;
	static char tmp[MaxStringSize] ;

	p = get_client_player(host,port) ;
	if(p == (Player *)NULL && iPlayer == 0) 
	{
		/* receive to server's port */
		sscanf(message,"(%s",tmp);
		if (!strcmp(tmp,"reconnect")) 
		{
			// a player reconnects to the server
			if (mode == PM_PlayOn)
				udp_send("(can't reconnect)", host, port, 0) ;
			else 
			{
				p = reconnectClient(host,port,message) ;
				if(p == (Player *)NULL)
					udp_send("(error reconnect)", host,port,0) ;
				else
					send_reconnect_info(*p) ;
			}
		}
		else 
		{
			// a new player connects to the server
			p = newClient(host,port,message) ;
			if(p == (Player *)NULL) 
				udp_send("(error no_more_team_or_player_or_goalie)", host,port,0) ;
			else
				send_init_info(*p) ;
		}
		write_log(*this, *p, message, RECV) ;
	}
	else if (p != (Player *)NULL && iPlayer != 0) 
	{
		/* receive to player's port */
		p->parse_command(message) ;
		write_log(*this, *p, message, RECV) ;
	}
	else 
	{
		if (verbose)
			cerr << "command error" << endl ;
	}
}

bool Stadium::ParseMonitorCommand(char *message, unsigned long host, int port)
{
	int n;

	if (!strcmp(message,"(dispinit)")) 
	{
		// a new monitor connected
		displist_t *p ;

		for (p = &top ; p->next != NULL ; p = p->next)
			/* NULL statement */ ;

		p->next = (displist_t *)malloc(sizeof(displist_t)) ;
		if (p->next == NULL) {
			perror("malloc") ;
			exit(1) ;
		}

		p->next->shost = host ;
		p->next->sport = port ;
		p->next->next = NULL ;
	}
	else if (!strcmp(message,"(dispstart)")) 
	{
		// kick off message
		_Start(*this) ;
	}
	else if (!strncmp(message,"(dispdiscard", 12)) 
	{
		// a player is discarded by the monitor
		int side, unum ;
		n = sscanf(message,"(dispdiscard %d %d)", &side, &unum) ;
		if (n == 2)
			discard_player(side, unum) ;
	}
	else 
	{
		// foul or drop_ball
		int x, y, side ;
		double real_x, real_y ;
		n = sscanf(message,"(dispfoul %d %d %d)", &x, &y, &side) ;
			
		if (n == 3) {
			real_x = x / SHOWINFO_SCALE ;
			real_y = y / SHOWINFO_SCALE ;
			if ( side == NEUTRAL )
				referee_drop_ball(real_x, real_y, side) ;
			else
				referee_get_foul(real_x, real_y, side) ;
		}
		else
			// no message sent by a soccermonitor
			return false;
	}
	return true;
}

void Stadium::udp_recv_from_coach()
{
	char message[MaxMesg] ;
	struct sockaddr_in cli_addr ;
	int n, len ;

	len = sizeof(cli_addr) ;
	n = recvfrom(coach->socketfd, message, MaxMesg, 0,
						(struct sockaddr *)&cli_addr, &len) ;

	if(n < 0) {
		if(WSAGetLastError() != WSAEWOULDBLOCK && WSAGetLastError() != WSAENOTSOCK)
			if (verbose)
				cerr << "recvfrom error" << endl ;
		return ;
	}
	coach->assignedp = True ;
	_Start(*this) ;

	/* chop newline */
	if(n != 0 && message[n-1] == '\n')
		n-- ;
	message[n] = NULLCHAR ;

	unsigned long host = ntohl(cli_addr.sin_addr.s_addr) ;
	int port = ntohs(cli_addr.sin_port) ;

	coach->parse_command(*this,message,host,port) ;
}

void Stadium::udp_recv_from_online_coach()
{
	static char message[MaxMesg] ;
	static char tmp[MaxStringSize] ;
	unsigned long host ;
	int i, m, n, r, len, port;
	OnlineCoach* p ;

	/* receive from socket for server(i=0) and online coachs(i=1, 2) */
	for ( i = 0 ; i < 3; i++) {
		if (i != 0) {
			if (olcoach[i]->assignedp == FALSE)
				continue ;
		}

		struct sockaddr_in cli_addr ;
		len = sizeof(cli_addr) ;

		n = 1 ; r = -1 ;

		while(n>=0) {
			m = n ;
			if (i == 0)		/* server socket */
				n = recvfrom(olcoachfd[0], message, MaxMesg, 0,
					(struct sockaddr *) &cli_addr, &len) ;
			else			/* client socket */
				n = recvfrom(olcoachfd[i], message, MaxMesg, 0,
					(struct sockaddr *) &cli_addr, &len) ;
			r++ ;
		}

		host = ntohl(cli_addr.sin_addr.s_addr) ;
		port = ntohs(cli_addr.sin_port) ;

		if (n == -1 && WSAGetLastError() == WSAEWOULDBLOCK) {
			if(r>0) {
				if(m == 0) continue ;
				/* chop newline */
				if(message[m-1] == '\n') m--;
				message[m] = NULLCHAR ;

				if(i == 0) {
					/* receive to server's port */
					n = sscanf(message,"(%s",tmp);
//					if (!strcmp(tmp,"reconnect")) {
//					}
					p = newCoach(host,port,message) ;
					if(p == (OnlineCoach *)NULL)
						udp_send("(error no_such_team_or_already_have_coach)",
								host,port,0, TRUE) ;
				}
				else {
					/* receive to coach's port */
					olcoach[i]->parse_command(*this,message,
							host,port) ;
				}
			}
		}
		else if(WSAGetLastError() != WSAENOTSOCK)
			if (verbose)
				cerr << "recvfrom error" << endl ;
	}

}

void chop_last_parenthesis(char *str, int max_size)
{
	int l = strlen(str) ;

	 if(l > max_size) {
		str[max_size] = NULLCHAR ;
	 }
	 else {
		l-- ;
		if(str[l] == ')') str[l] = NULLCHAR ;
	 }
}



void Player::parse_command(char *command)
{
	char com[MaxStringSize] ;
	int n ;
	double effective_dash_power ;

	for ( ; ; ) {
		if ((n = sscanf(command,"(%[^ )]",com)) < 1)
			break ;

		if(alive & DISCARD)
			return ;

		if(!strcmp(com,"dash")) {
			double power, power_need ;
			if ((n = sscanf(command,"(%s %lf)",com,&power)) < 2)
				break ;
			if (!command_done) {
				// culculate stamina
//				power = NormalizeDashPower(power) ;
				power_need = (power < 0) ? NormalizeDashPower(power) * -2 
								: NormalizeDashPower(power) ;
				if (power_need > stamina)
					power_need = stamina ;
				stamina -= power_need ;
				// 
				power = (power < 0) ? power_need / -2 : power_need ;
				effective_dash_power = effort * power ;
				dash(ReduceDashPower(effective_dash_power)) ;
				count_dash++ ;
				command_done = TRUE ;
			}
		}
		else if (!strcmp(com,"turn")) {
			double moment ;
			if ((n = sscanf(command,"(%s %lf)",com,&moment)) < 2)
				break ;
			if (!command_done) {
				turn(NormalizeMoment(moment)) ;
				count_turn++ ;
				command_done = TRUE ;
			}
		}
		else if (!strcmp(com,"turn_neck")) {
			double moment ;
			if ((n = sscanf(command,"(%s %lf)",com,&moment)) < 2)
				break ;
			if (!turn_neck_done) {
				turn_neck(NormalizeNeckMoment(moment)) ;
				count_turn_neck++ ;
				turn_neck_done = TRUE ;
			}
		}
		else if (!strcmp(com,"kick")) {
			double power,dir ;
			if ((n = sscanf(command,"(%s %lf %lf)",com,&power,&dir)) < 3)
				break ;
			if (!command_done) {
				kick(NormalizeKickPower(power),NormalizeMoment(dir)) ;
				count_kick++ ;
				command_done = TRUE ;
			}
		}
		else if (!strcmp(com,"catch")) {
			double dir ;
			if ((n = sscanf(command, "(%s %lf)",com,&dir)) < 2)
				break ;
			if (!command_done) {
				goalie_catch(NormalizeMoment(dir)) ;
				command_done = TRUE ;
			}
		}
		else if (!strcmp(com,"say")) {
			char msg[MaxStringSize] ;
			if ((n = sscanf(command,SAY_MESSAGE_SCAN_FORMAT,com,msg)) < 2)
				break ;
			chop_last_parenthesis(msg, stadium->SayMsgSize) ;
			say(msg) ;
			count_say++ ;
		}
		else if (!strcmp(com,"sense_body")) {
			sense_body() ;
		}
		else if (!strcmp(com,"move")) {
			double x, y ;
			/* (move X Y) */
			n = sscanf(command,"(%s %lf %lf)",com,&x,&y) ;
			if(n<3)
				/* (move (X Y)) */
				n = sscanf(command,"(%s (%lf %lf))",com,&x,&y) ;
			if(n<3)
				break ;
			if (!command_done) {
				if( stadium->mode == PM_BeforeKickOff ||
					stadium->mode == PM_AfterGoal_Right ||
#ifndef	_FREEKICK_MOVE_ALL_
					stadium->mode == PM_AfterGoal_Left) {
#else
					stadium->mode == PM_AfterGoal_Left ||
					stadium->mode == PM_FreeKick_Left ||
					stadium->mode == PM_FreeKick_Right ){
#endif
					pos.x = x * team->side ;
					pos.y = y * team->side ;
					stadium->check_collision(this) ;
				}
#ifndef	_FREEKICK_MOVE_ALL_
				else if ((stadium->mode == PM_FreeKick_Left ||
					stadium->mode == PM_FreeKick_Right) &&
					stadium->caught_ball == team->side && goalie) {
					pos.x = x * team->side ;
					pos.y = y * team->side ;
					stadium->check_collision(this) ;
				}
#endif
				else
					break ;
				command_done = TRUE ;
			}
		}
		else if (!strcmp(com, "change_view")) {
			char awd[16], qly[16] ;
			if ((n = sscanf(command, "(%s %s %[^ )]", com, awd, qly)) < 3)
				break ;

			if (!strcmp(awd, "narrow")) {
				vis_angle = defangle / 2 ;	
				vis_send = 2 ;
			}
			else if (!strcmp(awd, "normal")) {
				vis_angle = defangle ;
				vis_send = 4 ;
			}
			else if (!strcmp(awd, "wide")) {
				vis_angle = defangle * 2 ;
				vis_send = 8 ;
			}
			else
				break ;

			if (!strcmp(qly, "high"))
				highquality = TRUE ;
			else if (!strcmp(qly, "low")) {
				vis_send /= 2 ;
				highquality = FALSE ;
			}
			else
				break ;
		}
		else if (!strcmp(com, "bye")) {
			stadium->byeClient(this) ;
			command_done = TRUE ;
			return ;
		} 
		else {
			udp_send("(error unknown_command)") ;
			return ;
		}

		n = 0 ;
		for(;;) {
			if (*command == '(')
				n++ ;
			else if (*command == ')') {
				n-- ;
				if (n == 0) {
					command++ ;
					if (*command == NULLCHAR)
						return ;
					else
						break ;
				}
			}
			else if (*command == NULLCHAR) {
				return ;
			}
			command++ ;
		}
	}
	udp_send("(error illegal_command_form)") ;
	return ;
}

void Stadium::say(char *message)
{
	displist_t *p ;

	send_audio_info(message, NULL, FALSE) ;

	if (logfile[0] != NULLCHAR || rec_log || send_log) {
		minfo.body.msg.board = htons(MSG_BOARD) ;
		sprintf(minfo.body.msg.message, "(%s %s)\n", REFEREE_NAME, message) ;

		if (logfile[0] != NULLCHAR) {
			static char tmp[MaxStringSize] ;

			sprintf(tmp, "%d\t%s", time, minfo.body.msg.message ) ;
			fputs(tmp, log_fp) ;
			fflush(log_fp) ;
		}

		if (send_log)
			for (p = top.next ; p != NULL ; p = p->next)
				udp_send(&minfo,p->shost,p->sport,0) ;

		if (recfile[0] != NULLCHAR && rec_log == TRUE)
			if (mode != PM_BeforeKickOff && mode != PM_TimeOver)
				WriteLog(&minfo) ;
	}
}

void Player::say(char *message)
{
	Stadium &std = *(this->stadium) ;
	displist_t *p ;

	std.send_audio_info(message, this, FALSE) ;

	if (std.rec_log || std.send_log) {
		std.minfo.body.msg.board = htons(MSG_BOARD) ;
		sprintf(std.minfo.body.msg.message, "(%s %s)\n", name, message) ;

		if (std.send_log)
			for (p = std.top.next ; p != NULL ; p = p->next)
				std.udp_send(&(std.minfo),p->shost,p->sport,0) ;

		if (std.recfile[0] != NULLCHAR && std.rec_log == TRUE)
			if (std.mode != PM_BeforeKickOff && std.mode != PM_TimeOver)
				std.WriteLog(&std.minfo) ;
	}
}

void OnlineCoach::say(char *message)
{
	Stadium &std = *(this->stadium) ;
	displist_t *p ;

	std.send_audio_info(message, NULL, this->side) ;

	if (std.rec_log || std.send_log) {
		std.minfo.body.msg.board = htons(MSG_BOARD) ;
		sprintf(std.minfo.body.msg.message, "(%s %s)\n", 
			(side == RIGHT) ? OLCOACH_NAME_R : OLCOACH_NAME_L, message) ;

		if (std.send_log)
			for (p = std.top.next ; p != NULL ; p = p->next)
				std.udp_send(&(std.minfo),p->shost,p->sport,0) ;

		if (std.recfile[0] != NULLCHAR && std.rec_log == TRUE)
			if (std.mode != PM_BeforeKickOff && std.mode != PM_TimeOver)
				std.WriteLog(&std.minfo) ;
	}
}

void Stadium::send_audio_info(char *info, Player *id, int olcoach_side)
{
	int i,sw,buf ;
	char tmp[MaxStringSize] ;
	char tmp_name[MaxStringSize] ;
	ostrstream ost(tmp, MaxStringSize) ;
	PVector *source_pos ;

	/* randomize order */
	for(i = 0 ; i < MAX_PLAYER * 2; i++) {
		sw = irand(MAX_PLAYER * 2) ;
		buf = dest[i] ;
		dest[i] = dest[sw] ;
		dest[sw] = buf ;
	}

	if ( id == NULL) {
		if ( olcoach_side == FALSE ) {
			ost << "(hear " << time << " "
				<< REFEREE_NAME << " "
				<< info << ")" << endl << ends ;
			for(i = 0 ; i < MAX_PLAYER * 2; i++) {
				if (player[dest[i]]->alive == DISABLE)
					continue ;
				player[dest[i]]->send_audio_info(tmp) ;
			}
		}
		else {
			ost << "(hear " << time << " "
				<< ((olcoach_side == RIGHT) ? OLCOACH_NAME_R : OLCOACH_NAME_L )<< " "
				<< info << ")" << endl << ends ;
			for(i = 0 ; i < MAX_PLAYER * 2; i++) {
				if (player[dest[i]]->alive == DISABLE)
					continue ;
				player[dest[i]]->send_audio_info(tmp) ;
			}
		}
	}
	else {
		for(i = 0 ; player[dest[i]] != id ; i++ ) ;
		source_pos = &(player[dest[i]]->pos) ;

		for(i = 0 ; i < MAX_PLAYER * 2; i++ ) {
			if (player[dest[i]]->alive == DISABLE)
				continue ;
			if (distance(source_pos, &(player[dest[i]]->pos)) <= audio_dist) {
				if (player[dest[i]] == id) {
					sprintf(tmp, "(hear %d self ", time) ;
					strcat(tmp, info) ;
					strcat(tmp, ")\n") ;
					player[dest[i]]->send_audio_info(tmp) ;
				}
				else {
					sprintf(tmp, "(hear %d %d ", time,
						Rad2IDeg(player[dest[i]]->vangle(*id))) ;
					strcat(tmp, info) ;
					strcat(tmp, ")\n") ;

					if (player[dest[i]]->team == id->team) {
						if (player[dest[i]]->hear_capacity_from_teammate
															>= hear_decay) {
							player[dest[i]]->send_audio_info(tmp) ;
							player[dest[i]]->hear_capacity_from_teammate
															-= hear_decay ;
						}
					}
					else {
						if (player[dest[i]]->hear_capacity_from_opponent
															>= hear_decay) {
							player[dest[i]]->send_audio_info(tmp) ;
							player[dest[i]]->hear_capacity_from_opponent
															-= hear_decay ;
						}
					}
				}
			}
		}
	}

    if ( id == NULL ){
		switch( olcoach_side ){
			case RIGHT:
				strcpy(tmp_name,OLCOACH_NAME_R);
				break;
			case LEFT:
				strcpy(tmp_name,OLCOACH_NAME_L);
				break;
			default:
				strcpy(tmp_name,REFEREE_NAME);
		}
	}else{
		strcpy(tmp_name,((Player *)id)->name);
	}

	if ((CoachMode || CwRMode) && coach->hear) {
		if (old_hear)	/* old format */
			sprintf(tmp, "(hear %s %d %s)\n", tmp_name, time, info) ;
		else			/* new format (same as player's) */
			sprintf(tmp, "(hear %d %s %s)\n", time, tmp_name, info) ;
		coach->udp_send(tmp) ;
	}
	

	for (int ii = 1; ii < 3; ii++) {
		if (olcoach[ii]->assignedp) {
			sprintf(tmp, "(hear %d %s %s)\n", time, tmp_name, info) ;
			olcoach[ii]->udp_send(tmp) ;
		}
	}
	
}

void Player::send_audio_info(char *message)
{
	udp_send(message,FALSE) ;
}


/*
 *===================================================================
 *  Visual Information Loop
 *===================================================================
 */
void Stadium::udp_send_message() 
{
	int i, sw, tmp ;

	/* send to clients */
	for(i = 0 ; i < MAX_PLAYER * 2; i++) {
		sw = irand(MAX_PLAYER * 2) ;
		tmp = dest[i] ;
		dest[i] = dest[sw] ;
		dest[sw] = tmp ;
	}

	for(i = 0 ; i < MAX_PLAYER * 2; i++ ) {
		if (player[dest[i]]->alive == DISABLE)
			continue ;
		player[dest[i]]->send_visual_info() ;
	}
}

void Player::send_visual_info(void) 
{
	int i, n ;
	static char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;

	sendcnt++ ;

	if (sendcnt >= vis_send)
		sendcnt = 0 ;
	else
		return ;

	ost << "(see " << stadium->time; 

	n = scanField() ;
	for( i = 0 ; i < n ; i++ ) {
		if (vi[i].object->obj_ver <= version)
			ost << " " << vi[i] ;
	}

	ost << ")" << endl << ends ;

	udp_send(tmp,FALSE) ;
}

/*
 *===================================================================
 *  Reply sense_body
 *===================================================================
 */
void Player::sense_body(void)
{
	static char tmp[MaxStringSize] ;
	static char quality[8] ;
	static char width[8] ;
	ostrstream ost(tmp,MaxStringSize) ;

	strcpy(quality, (highquality) ? "high" : "low") ;

	if (vis_angle == defangle/2)
		strcpy(width, "narrow") ;
	else if (vis_angle == defangle*2)
		strcpy(width, "wide") ;
	else
		strcpy(width, "normal") ;

	if( version >= 5.0 ){
		ost << "(sense_body " << stadium->time
			<< " (view_mode " << quality << " " << width
			<< ") (stamina " << stamina << " " << effort
			<< ") (speed " << Quantize(vel.r(), 0.01)
			<< ") (head_angle " << 	Rad2IDeg(angle_neck_committed)
			<< ") (kick " << count_kick
			<< ") (dash " << count_dash
			<< ") (turn " << count_turn
			<< ") (say " << count_say
			<< ") (turn_neck " << count_turn_neck
			<< "))"
			<< endl << ends ;
	}else{
		ost << "(sense_body " << stadium->time
			<< " (view_mode " << quality << " " << width
			<< ") (stamina " << stamina << " " << effort
			<< ") (speed " << Quantize(vel.r(), 0.01)
			<< ") (kick " << count_kick
			<< ") (dash " << count_dash
			<< ") (turn " << count_turn
			<< ") (say " << count_say
			<< "))"
			<< endl << ends ;
	}

	udp_send(tmp,FALSE) ;
}

/*
 *===================================================================
 *  UDP Send
 *===================================================================
 */
void Stadium::udp_send(const char *msg, Player &player,const Logical logp)
{
	if (player.client.port > 0) {
		udp_send(msg,player.client.host,player.client.port,
			((((player.team)->side) == LEFT) ? 0 : 1) * MAX_PLAYER + player.unum) ;
		if(logp)
			write_log(*this,player,msg,SEND) ;
	}
}

void Stadium::udp_send(const char *msg, unsigned long host, int port,int fd)
{
	struct sockaddr_in cli_addr ;
	int n ;

	bzero((char *) &cli_addr, sizeof(cli_addr)) ;
	cli_addr.sin_family			= AF_INET ;
	cli_addr.sin_addr.s_addr	= htonl(host) ;
	cli_addr.sin_port			= htons(port) ;

	n = strlen(msg) ;
	if (socketfd[fd] != -1)
		if(sendto(socketfd[fd], (char *)msg, n, 0,
					(struct sockaddr *)&cli_addr, sizeof(cli_addr)) != n){
#ifdef DEBUG
			if (verbose)
				cerr << "sendto error on socket." << endl ;
#endif
		}
}

void Stadium::udp_send(const char *msg, unsigned long host, int port,int fd, int online)
{
	struct sockaddr_in cli_addr ;
	int n ;

	bzero((char *) &cli_addr, sizeof(cli_addr)) ;
	cli_addr.sin_family		= AF_INET ;
	cli_addr.sin_addr.s_addr	= htonl(host) ;
	cli_addr.sin_port		= htons(port) ;

	n = strlen(msg) ;
	if (olcoachfd[fd] != -1)
		if(sendto(olcoachfd[fd], (char *)msg, n, 0,
					(struct sockaddr *)&cli_addr, sizeof(cli_addr)) != n){
#ifdef DEBUG
			if (verbose)
				cerr << "sendto error on socket." << endl ;
#endif
		}
}

void Stadium::udp_send(dispinfo_t *msg, unsigned long host, int port,int fd)
{
	struct sockaddr_in cli_addr ;
	int n ;

	bzero((char *) &cli_addr, sizeof(cli_addr)) ;
	cli_addr.sin_family			= AF_INET ;
	cli_addr.sin_addr.s_addr	= htonl(host) ;
	cli_addr.sin_port			= htons(port) ;

	n = sizeof(dispinfo_t) ;
	if (socketfd[fd] != -1)
		if(sendto(socketfd[fd], (char *)msg, n, 0,
					(struct sockaddr *) &cli_addr, sizeof(cli_addr)) != n){
#ifdef DEBUG
			if (verbose)
				cerr << "sendto error on socket." << endl ;
#endif
		}
}

static char *PlayModeString[] = PLAYMODE_STRINGS ;

PlayMode PlayModeID(char *mode)
{
	int n ;
	for(n = 0 ; n < PM_MAX ; n++) {
		if(!strcmp(PlayModeString[n],mode))
			return (PlayMode)n ;
	}
	return PM_Null ;
}

void Stadium::send_init_info(Player& player) 
{
	char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;

	ost << "(init " << SideStr(player.team->side) << " " 
		<< player.unum << " "
		<< PlayModeString[mode]
		<< ")" << endl << ends ;
	udp_send(tmp,player) ;
}

void Stadium::send_reconnect_info(Player& player) 
{
	char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;

	ost << "(reconnect " << SideStr(player.team->side) << " " 
		<< PlayModeString[mode]
		<< ")" << endl << ends ;
	udp_send(tmp,player) ;
}

void Player::udp_send(const char *message,const Logical logp)
{
	stadium->udp_send(message,*this,logp) ;
}

/*
 *===================================================================
 *  Coach
 *===================================================================
 */
void Coach::parse_command(Stadium& std, char *command, 
							unsigned long host, int port)
{
	client.host = host ;
	client.port = port ;

	char com[MaxStringSize] ;
	int n ;

	n = sscanf(command, "(%[-0-9a-zA-Z.+*/?<>_]",com) ;
	if(n<1) {udp_send("(error illegal_command_form)"); return ; }

	if(!strcmp(com,"change_mode")) {
		char new_mode[MaxStringSize] ;
		PlayMode mode_id ;
		n = sscanf(command,"(change_mode %[-0-9a-zA-Z.+*/?<>_])", new_mode) ;
		if(n<1) {udp_send("(error illegal_command_form)"); return ;}
		mode_id = PlayModeID(new_mode) ;

		if(mode_id == PM_Null) { udp_send("(error illegal_mode)") ; return ; }
		std.change_play_mode(mode_id) ; 
		udp_send("(ok change_mode)") ;
	}
	else if (!strcmp(com,"move")) {
		char obj[MaxStringSize] ;
		double x,y,ang, velx,vely ;

		command++ ;
		while(*command != '(') {
			if (*command == NULLCHAR) {
				udp_send("(error illegal_object_form)") ;
				return ;
			}
			command++ ;
		}
		int i = 0 ;
		while(*command != ')') {
			if (*command == NULLCHAR) {
				udp_send("(error illegal_object_form)") ;
				return ;
			}
			*(obj+i) = *command ;
			i++ ; command++ ;
		}
		*(obj+i) = *command ;
		i++ ; command++ ;
		*(obj+i) = NULLCHAR ;

		if (!strcmp(obj, BALL_NAME))
			std.caught_ball = FALSE;

		n = sscanf(command," %lf %lf %lf %lf %lf)",&x,&y,&ang,&velx,&vely) ;
		if (n==2) {
			if (!strcmp(obj, BALL_NAME))
				std.set_ball(PVector(x,y), LEFT) ;
			else
				std.move_object(obj,PVector(x,y)) ;
			std.check_collision(obj) ;
		}
		else if (n==3) {
			if (!strcmp(obj, BALL_NAME))
				std.set_ball(PVector(x,y), LEFT, ang) ;
			else
				std.move_object(obj,PVector(x,y),
					Deg2Rad(mmax(mmin((ang),std.maxm),std.minm))) ;
			std.check_collision(obj) ;
		}
		else if (n==5) {
			if (!strcmp(obj, BALL_NAME))
				std.set_ball(PVector(x,y), LEFT, ang, PVector(velx, vely)) ;
			else
				std.move_object(obj,PVector(x,y),Deg2Rad(mmax(mmin((ang),std.maxm),std.minm)),
					PVector(velx, vely));
			std.check_collision(obj) ;
		}
		else {
			udp_send("(error illegal_command_form)");
			return ;
		}
		udp_send("(ok move)") ;
	}
	else if (!strcmp(com,"look")) {
		look(std) ;
	}
	else if (!strcmp(com,"check_ball")) {
		check_ball(std) ;
	}
	else if (!strcmp(com,"say")) {
		static char msg[MaxStringSize] ;
		n = sscanf(command,SAY_MESSAGE_SCAN_FORMAT,com,msg) ;
		if(n<2) {udp_send("(error illegal_command_form)"); return ;}
		chop_last_parenthesis(msg, std.SayMsgSize) ;
		std.say(msg) ;
		udp_send("(ok say)") ;
	}
	else if (!strcmp(com,"ear")) {
		char mode[16] ;
		n = sscanf(command,"(%s %s)", com, mode) ;
		if(n<2) {udp_send("(error illegal_command_form)"); return ;}
		if (!strcmp(mode,"on)")) {
			hear = TRUE ;
			udp_send("(ok ear on)") ;
		}
		else if (!strcmp(mode, "off)")) {
			hear = FALSE ;
			udp_send("(ok ear off)") ;
		}
	}
	else if (!strcmp(com,"eye")) {
		char mode[16] ;
		n = sscanf(command,"(%s %s)", com, mode) ;
		if(n<2) {udp_send("(error illegal_command_form)"); return ;}
		if (!strcmp(mode,"on)")) {
			eye = TRUE ;
			udp_send("(ok eye on)") ;
		}
		else if (!strcmp(mode, "off)")) {
			eye = FALSE ;
			udp_send("(ok eye off)") ;
		}
	}
	else {
		udp_send("(error unknown_command)") ;
		return ;
	}
}

void Coach::look(Stadium& std) 
{
	static char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;

	ost << "(ok look " << std.time ;

	ost << " (" << GOAL_L_NAME 
		<< " " << -(PITCH_LENGTH/2.0)
		<< " " << 0.0
		<< ")" ;
	ost << " (" << GOAL_R_NAME 
		<< " " << (PITCH_LENGTH/2.0)
		<< " " << 0.0
		<< ")" ;
	ost << " (" << BALL_NAME
		<< " " << std.ball->pos.x
		<< " " << std.ball->pos.y
		<< " " << std.ball->vel.x
		<< " " << std.ball->vel.y
		<< ")" ;

	TheNumber i ;
	for(i = 0 ; i < MAX_PLAYER * 2; i++ ) {
		if (std.player[i]->alive == DISABLE)
			continue ;
		ost << " (" << (std.player[i])->name
			<< " " << (std.player[i])->pos.x
			<< " " << (std.player[i])->pos.y
			<< " " << Rad2IDeg((std.player[i])->angle)
			<< " " << Rad2IDeg((std.player[i])->angle_neck_committed)
			<< " " << (std.player[i])->vel.x
			<< " " << (std.player[i])->vel.y
			<< ")" ;
	}
	ost << ")" << endl << ends;

	udp_send(tmp) ;
}

void Coach::send_visual_info(void) 
{
	int i;
	static char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;
	double version = 5.0 ;

	ost << "(see " << stadium->time; 

	ost << " (" << GOAL_L_NAME 
		<< " " << -(PITCH_LENGTH/2.0)
		<< " " << 0.0
		<< ")" ;
	ost << " (" << GOAL_R_NAME 
		<< " " << (PITCH_LENGTH/2.0)
		<< " " << 0.0
		<< ")" ;
	ost << " (" << BALL_NAME
		<< " " << stadium->ball->pos.x
		<< " " << stadium->ball->pos.y
		<< " " << stadium->ball->vel.x
		<< " " << stadium->ball->vel.y
		<< ")" ;

	for(i = 0 ; i < MAX_PLAYER * 2; i++ ) {
		if (stadium->player[i]->alive == DISABLE)
			continue ;
		ost << " (" << (stadium->player[i])->name
			<< " " << (stadium->player[i])->pos.x
			<< " " << (stadium->player[i])->pos.y
			<< " " << Rad2IDeg((stadium->player[i])->angle)
			<< " " << Rad2IDeg((stadium->player[i])->angle_neck_committed)
			<< " " << (stadium->player[i])->vel.x
			<< " " << (stadium->player[i])->vel.y
			<< ")" ;
	}
	ost << ")" << endl << ends;

	udp_send(tmp) ;
}

void Coach::check_ball(Stadium& std)
{
	static char tmp[MaxStringSize] ;
	ostrstream ost(tmp,MaxStringSize) ;

	static char* BallPosInfoStr[] = BALL_POS_INFO_STRINGS ;
	BallPosInfo info = std.check_ball_pos() ;

	ost << "(ok check_ball " << std.time << " " ;

	ost << BallPosInfoStr[info] << ")" << ends ;

	udp_send(tmp) ;
}

void Coach::udp_send(const char *msg)
{
	struct sockaddr_in cli_addr ;
	int n ;

	bzero((char *) &cli_addr, sizeof(cli_addr)) ;
	cli_addr.sin_family		= AF_INET ;
	cli_addr.sin_addr.s_addr	= htonl(client.host) ;
	cli_addr.sin_port		= htons(client.port) ;

	n = strlen(msg) ;
	if(sendto(socketfd, (char *)msg, n, 0,
			(struct sockaddr *) &cli_addr, sizeof(cli_addr)) != n){
#ifdef DEBUG
		if (verbose)
			cerr << "sendto error on socket." << endl ;
#endif
	}
}

void OnlineCoach::parse_command(Stadium& std, char *command, 
							unsigned long host, int port)
{
	char com[MaxStringSize] ;
	int n;

	n = sscanf(command, "(%[-0-9a-zA-Z.+*/?<>_]",com) ;
	if(n<1) {udp_send("(error illegal_command_form)"); return ; }


	if (!strcmp(com,"check_ball")) {
		check_ball(std) ;
	}
	else if (!strcmp(com,"look")) {
		look(std) ;
	}
	else if (!strcmp(com,"say")) {
		if(std.mode != PM_PlayOn) {
			static char msg[MaxStringSize] ;
			n = sscanf(command,SAY_MESSAGE_SCAN_FORMAT,com,msg) ;
			if(n<2) {
				udp_send("(error illegal_command_form)");
				return ;
			}
			chop_last_parenthesis(msg, std.SayCoachMsgSize) ;
			say(msg) ;
			udp_send("(ok say)") ;
		}
		else {
			udp_send("(warning cannot_say_while_playon)");
			return ;
		}
	}
	else if (!strcmp(com, "bye")) {
		assignedp = FALSE;
		client.host = 0 ;
		client.port = -1 ;
		socketfd = -1 ;
		return ;
	} 
	else if (!strcmp(com,"eye")) {
		char mode[16] ;
		n = sscanf(command,"(%s %s)", com, mode) ;
		if(n<2) {udp_send("(error illegal_command_form)"); return ;}
		if (!strcmp(mode,"on)")) {
			eye = TRUE ;
			udp_send("(ok eye on)") ;
		}
		else if (!strcmp(mode, "off)")) {
			eye = FALSE ;
			udp_send("(ok eye off)") ;
		}
	}
	else {
		udp_send("(error unknown_command)") ;
		return ;
	}
}



/*
 *===================================================================
 *  Misc
 *===================================================================
 */
void write_log(Stadium& std, Player& p, const char *message, int flag) 
{
	displist_t *dp ;
	static char tmp[MaxStringSize] ;

	if (std.rec_log || std.send_log) {
		std.minfo.body.msg.board = htons(LOG_BOARD) ;

		if (&p == NULL)
			sprintf(tmp, "%s", "Illegal\n") ;
		else
			sprintf(tmp, "%s %s_%d: %s\n", (flag == SEND) ? "Send" : "Recv", 
							p.team->name, p.unum, message) ;

		strncpy(std.minfo.body.msg.message, tmp, 128) ;

		if (std.logfile[0] != NULLCHAR) {
			static char tmp2[MaxStringSize] ;

			sprintf(tmp2, "%d\t%s", std.time, tmp ) ;
			fputs(tmp2, std.log_fp) ;
			fflush(std.log_fp) ;
		}

		if (std.send_log)
			for (dp = std.top.next ; dp != NULL ; dp = dp->next)
				std.udp_send(&(std.minfo),dp->shost,dp->sport,0) ;

		if (std.recfile[0] != NULLCHAR && std.rec_log == TRUE)
			if (std.mode != PM_BeforeKickOff && std.mode != PM_TimeOver)
				std.WriteLog(&std.minfo) ;
	}
}


Value distance(PVector *pos1, PVector *pos2)
{
	return sqrt(Pow(pos2->x - pos1->x) + Pow(pos2->y - pos1->y)) ;
}
