/*
 *Header:
 *File: object.h (for C++ & cc)
 *Author: Noda Itsuki
 *Date: 1995/02/21
 *EndHeader:
 */

/*
 *ModifyHistory
 *
 *EndModifyHistory
 */

#ifdef __cplusplus

#define SEPLINE	"--------------------"

inline Value square(Value x) { return x * x ; }

//extern ostream& operator<< (ostream& o, const Angle& ang) ;

/*
 *===================================================================
 *Part: Plain Vector
 *===================================================================
 */
class PVector {
 public:
	Value	x ;
	Value	y ;

	PVector(const double vx = 0.0, const double vy = 0.0) ;
	void	operator +=(const PVector& v) ;
	void	operator -=(const PVector& v) ;
	void	operator *=(const Value& a) ;
	void	operator /=(const Value& a) ;
	inline Value r() { return sqrt(square(x)+square(y)) ;} ;
	inline Value th() { return Atan(y,x) ; } ;
	friend PVector operator +(const PVector& a, const PVector& b) ;
	friend PVector operator -(const PVector& a, const PVector& b) ;
	void normalize(const Value& l = 1.0) ;
	Value distance(const PVector& orig) ;
	Value angle() ;
	Value angle(const PVector& dir) ;
	void rotate(const Angle& ang) ;
	Value vangle(const PVector& target, const PVector& origin) ;
	Value vangle(const PVector& target, const Angle& origin) ;
} ;

inline PVector Polar2PVector(Value r, Angle ang)
{
	return PVector(r * cos(ang), r * sin(ang)) ;
}

#include <iostream.h>
extern ostream& operator<< (ostream& o, const PVector& v) ;

/*
 *===================================================================
 *Part: Area
 *===================================================================
 */
class RArea {
 public:
	Value left ;
	Value right ;
	Value top ;
	Value bottom ;

	RArea(const Value left, const Value right, 
				const Value top, const Value bottom) ;
	RArea(const PVector center,const PVector size) ;

	Logical inArea(const PVector& p) ;
	PVector nearestHEdge(const PVector& p) ;
	PVector nearestVEdge(const PVector& p) ;
	PVector nearestEdge(const PVector& p) ;
	PVector randomize() ;

} ;

extern ostream& operator<< (ostream& o, const RArea& a) ;

class CArea {
 public:
	PVector center ;
	Value r ;

	CArea(const PVector center,const Value r) ;

	Logical inArea(const PVector& p) ;
	PVector nearestEdge(const PVector& p) ;
} ;


extern ostream& operator<< (ostream& o, const CArea& a) ;

/*
 *===================================================================
 *Part: Visual Info
 *===================================================================
 */
class PObject ;

class VisualInfo {
 public:
	PObject *object ;
	Name name ;
	Value distance ;
	Angle direction ;
	Value dist_change ;
	Angle dir_change ;
	Angle face_dir ;
	Angle body_dir ;
	int highquality ;
	Value qstep ;
	Value version ;

	VisualInfo(double) ;
	VisualInfo() ;
	void set(double) ;
} ;

extern ostream& operator<< (ostream& o, const VisualInfo& vi) ;

/*
 *===================================================================
 *Part: PObject
 *===================================================================
 */
class Stadium ;

class PObject {
 protected:
	static TheNumber Max ;
 public:
	ID		id ;
	Name	name ;
	PVector	pos ;
	Value	obj_ver ;
	Value	size ;
	Angle	angle ;
	int		enable ;
	Stadium	*stadium ;
	VisualInfo *vi ;

	PObject(const Name name=NULLCHAR,
					const PVector& p=PVector(0.0,0.0), const Value v=3.0) ;
	void Set(const PVector& p, const Value& s) ; 
	Angle vangle(const PObject& obj) ;
	void reportScanField(ostream& o) ;
} ;

extern ostream& operator<< (ostream& o, const PObject& v) ;

class MPObjectTable ;
class Weather ;

class MPObject : public PObject {
 public:
	PVector	vel ;
	PVector	acc ;
	Value	decay ;
	Value	randp ;
	Value	weight ;
	Value	max_speed ;
	Weather *weather ;
	MPObjectType type ;

	MPObject(const Name name=NULLCHAR) ;
	inline void	operator ++() { _inc(); } ;
	void _inc() ;
        void _turn() ;
        
	PVector noise() ;
	PVector wind() ;
	void Set(const MPObjectType& t, const PVector& p, const Value& s,
				const Angle& ang, const PVector& v, const PVector& a,
				const Value& d, const Value& r=0.0) ;
	Logical collisionP(const MPObject& obj) ;
	void collisionCheck(MPObjectTable& tbl) ;
	void collide(MPObject& obj) ;
	void step(MPObjectTable& tbl) ;
	void push(PVector f) ;
	void rotate(Angle a) ;
	void rotate_to(Angle dir) ;
	void dash(Value f) ;
} ;

extern ostream& operator<< (ostream& o, const MPObject& v) ;

/*
 *===================================================================
 *Part: Object Table
 *===================================================================
 */

class PObjectTable {
 public:
	TheNumber	Max ;
	TheNumber	n ;
	PObject** object ;

	PObjectTable(const TheNumber max = 128) ;
	~PObjectTable() ;
	PObject* AssignObject(PObject* obj) ;
	PObject* NewObject(const Name name=NULLCHAR) ;
} ;
extern ostream& operator<< (ostream& o, const PObjectTable& tbl) ;

class MPObjectTable {
	void _inc() ;
 public:
	TheNumber	Max ;
	TheNumber	n ;
	MPObject** object ;

	MPObjectTable(const TheNumber max = 128) ;
	~MPObjectTable() ;
	MPObject* AssignObject(MPObject* obj) ;
	MPObject* NewObject(const Name name=NULLCHAR) ;
	inline void	operator++ () { _inc(); } ;
        void _turn();
        
	/*inline void	operator ++(int dummy) { _inc(); } ;*/
	void collisionCheck(MPObject* obj) ;
} ;
extern ostream& operator<< (ostream& o, const MPObjectTable& tbl) ;

/*
 *===================================================================
 *Part: Player & Team
 *===================================================================
 */

class Client {
 public:
	unsigned long host ;
	int	port ;
} ;

class Team ;

class Player : public MPObject {
 public:
	Team *team ;
	ID	unum ;		/* uniform number */
	Name unumstr ;
	Client client ;
	Name name_far ;
	Name name_toofar ;
	Value stamina ;
	Value recovery ;
	Value effort ;
	Value vis_angle ;
	Value defangle ;
	Value vis_distance ;
	Value inertia_moment ;
	Value version ;

	Angle angle_neck ;
#if 1
	Angle angle_neck_committed ;
#endif
	Angle angle_body ;

	int vis_send ;
	int sendcnt ;
	int highquality ;
	int command_done ;
	int turn_neck_done ;
	int alive ;
	int hear_capacity_from_teammate ;
	int hear_capacity_from_opponent ;
	int goalie ;
	int goalie_catch_ban ;
	Value goalie_catch_probability ;
	int	count_kick ;
	int count_dash ;
	int count_turn ;
	int count_turn_neck ;
	int count_say ;
	int offside_mark ;
	PVector offside_pos ;
	PVector relative_ball_ang ;
	
	Player(Team *team, Stadium *std, ID number) ;
	void init(Value version, int goalie_flag) ;
	void rotate(Angle a) ;
	void rotate_to(Angle a) ;
	void turn(Value a) ;
	void turn_neck(Value a) ;
	void kick(Value power, Angle dir) ;
	void goalie_catch(Angle dir) ;

	void parse_command(char *command) ;
	int scanField(void) ;
	void send_visual_info(void) ;
	void send_audio_info(char *message) ;	
	void say(char *message) ;
	void sense_body(void) ;
	void udp_send(const char *message,
			const Logical logp=TRUE) ;
} ;

class OnlineCoach ;

class Team {
 public:
	Stadium	*stadium ;
	int enable ;
	Name name ;
	Side side ;
	Name color ;
	TheNumber point ;

	TheNumber Max ;
	TheNumber n ;
	int goalie ;
	Player **player ;

	Team(Stadium *std, const Side side) ;
	Player *newPlayer(Value version, int goalie_flag) ;
	OnlineCoach *olcoach ;
} ;

#endif
